# -*- coding: utf-8 -*-
"""
本模块功能：中国基金市场
所属工具包：证券投资分析工具SIAT 
SIAT：Security Investment Analysis Tool
创建日期：2020年10月17日
最新修订日期：2020年10月21日
作者：王德宏 (WANG Dehong, Peter)
作者单位：北京外国语大学国际商学院
版权所有：王德宏
用途限制：仅限研究与教学使用，不可商用！商用需要额外授权。
特别声明：作者不对使用本工具进行证券投资导致的任何损益负责！
"""

#==============================================================================
#关闭所有警告
import warnings; warnings.filterwarnings('ignore')
from siat.common import *
from siat.translate import *
from siat.grafix import *
from siat.bond_base import *
#==============================================================================
if __name__=='__main__':
    fund='180801'
    quarter1='2020Q1'
    quarter2='2021Q2'

#比较两个季度之间的基金持仓变化
def fund_stock_holding_compare_china(fund,quarter1,quarter2,top=10):
    """
    功能：基金fund在两个季度quarter1和quarter2的持仓股票对比（股数和金额），前top名股票
    参数：
    fund,str,基金代码;
    quarter1,str,靠前的季度, 格式为 'YYYYQ1',例如: '2021Q2';
    quarter2,str,靠后的季度, 格式为 'YYYYQ1',例如: '2021Q2';
    """
    print("Searching fund stock holding info, it may take time, please wait ...\n")
    
    import akshare as ak
    import pandas as pd    
    
    code=fund
    s1=quarter1.upper()
    s2=quarter2.upper()
    years=[s1[0:4],s2[0:4]]

    s1_share = s1+'持股数'
    s2_share = s2+'持股数'
    s1_value = s1+'持仓市值'
    s2_value = s2+'持仓市值'
    s1_ratio = s1+'持仓比例'
    s2_ratio = s2+'持仓比例'

    try:
        data = ak.fund_em_portfolio_hold(code=fund,year=years[0])
    except:
        print("  #Error(fund_stock_holding_compare_china): stock fund",fund,"not found or wrong year",years[0])
        return
    if len(data)==0:
        print("  #Error(fund_stock_holding_compare_china): stock fund",fund,"not found or wrong year",years[0])
        return      
    
    for yr in years:
        try:
            df_tmp = ak.fund_em_portfolio_hold(code=code,year=yr)
        except:
            print("  #Error(fund_stock_holding_compare_china): wrong year",yr)
            return            
        data = data.append(df_tmp)
    data.drop_duplicates(keep='first', inplace=True)

    data['季度']=data['季度'].apply(lambda x:x[:6])
    data['季度'] = data['季度'].str.replace('年','Q')
    data['占净值比例'] = pd.to_numeric(data['占净值比例'])

    df1 =data[data['季度']==s1]
    if len(df1)==0:
        print("  #Error(fund_stock_holding_compare_china): no data available for",s1)
        return        
    
    df1 = df1[['股票代码', '股票名称','持股数','持仓市值','占净值比例']]
    df1 = df1.rename(columns={'持股数':s1_share,'持仓市值':s1_value,'占净值比例':s1_ratio})
    df2 =data[data['季度']==s2]
    if len(df2)==0:
        print("  #Error(fund_stock_holding_compare_china): no data available for",s2)
        return 
    
    df2 = df2[['股票代码', '股票名称','持股数','持仓市值','占净值比例']]
    df2 = df2.rename(columns={'持股数':s2_share,'持仓市值':s2_value,'占净值比例':s2_ratio})

    df_merge = pd.merge(df1,df2,on='股票代码',how='outer')

# Q2 和 Q4，即半年度和年度报告，是需要披露全部持仓的
# 合并后，在dataframe 中 NaN 的数据应为 0

    if s1.endswith('Q2') or s1.endswith('Q4'):
        df_merge[s1_share] = df_merge[s1_share].fillna(0)
        df_merge[s1_value] = df_merge[s1_value].fillna(0)
        df_merge[s1_ratio] = df_merge[s1_ratio].fillna(0)

    if s2.endswith('Q2') or s2.endswith('Q4'):
        df_merge[s2_share] = df_merge[s2_share].fillna(0)
        df_merge[s2_value] = df_merge[s2_value].fillna(0)
        df_merge[s2_ratio] = df_merge[s2_ratio].fillna(0)

    df_merge['持股数变化'] = df_merge[s2_share] - df_merge[s1_share]
    df_merge = df_merge.sort_values(s2_value,ascending=False)
    
    df_merge['股票名称'] = df_merge['股票名称_y']
    df_merge.loc[df_merge['股票名称'].isna(),'股票名称'] = df_merge.loc[df_merge['股票名称'].isna(),'股票名称_x']
    df_merge = df_merge[['股票代码', '股票名称', s1_share,s1_value, s1_ratio,s2_share,s2_value,s2_ratio, '持股数变化']]
    
    df_merge.reset_index(drop=True,inplace=True)
    if top>0:
        df=df_merge.head(top)
    else:
        df=df_merge.head(-top)
    """
    #持股数和持仓比例取整数
    df.fillna(0)
    try:
        df[s1_share]=df[s1_share].astype('int')
    except: pass
    try:
        df[s2_share]=df[s2_share].astype('int')
    except: pass
    try:
        df[s1_value]=df[s1_value].astype('int')
    except: pass
    try:
        df[s2_value]=df[s2_value].astype('int')
    except: pass
    df['持股数变化'] = df[s2_share] - df[s1_share]
    """
    #设置打印对齐
    pd.set_option('display.max_columns', 1000)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)

    #获取基金名称
    names = ak.fund_em_fund_name()
    namedf=names[names['基金代码']==fund]
    if len(namedf)==0:
        name=fund
    else:
        name=namedf['基金简称'].values[0]
        
    order='前'
    if top <0: 
        order='后'
        top=-top
    print("===== 中国基金持仓股票分析："+name+'，'+s1+"对比"+s2,"(按后者持仓比例高低排列，"+order+str(top)+"名股票) =====\n")
    print(df.to_string(index=False))
    import datetime; today = datetime.date.today()
    print("\n*** 注：持股数为万股，持仓市值为万元，持仓比例为占基金资产净值比例%，包括A股与非A股")
    print("    数据来源：天天基金/东方财富, 期间持仓股票总计"+str(len(df_merge))+"只,",today)      
    
    return df_merge

#==============================================================================

if __name__=='__main__':
    fund='005827'
    year_num=2
    
# 获取单只基金的十大股票名称信息
def fund_stock_holding_rank_china(fund,year_num=2):
    """
    比较股票型基金fund近year_num年持仓的前10大股票排名变化
    """
    print("Searching fund stock holding info, it may take time, please wait ...\n")
    code=fund
    
    import akshare as ak
    import pandas as pd

    import datetime; today = datetime.date.today()
    year_0_num=int(str(today)[0:4])
    years=[]
    for yr in range(0,year_num):
        yri=str(year_0_num - yr)
        years=years+[yri]
    years.sort(reverse=False)    
    
    #抓取第一年的信息
    data = ak.fund_em_portfolio_hold(code=fund,year=years[0])
    if len(data)==0:
        print("  #Error(fund_stock_holding_rank_china): stock fund",fund,"not found")
        return          
    
    for yr in years:
        df_tmp = ak.fund_em_portfolio_hold(code=code,year=yr)
        data = data.append(df_tmp)
    data.drop_duplicates(keep='first', inplace=True)

    # data['季度']=data['季度'].apply(lambda x:x[:8])
    data['季度']=data['季度'].apply(lambda x:x[:6])
    data['季度'] = data['季度'].str.replace('年','Q')
    data['占净值比例'] = pd.to_numeric(data['占净值比例'])
    
    # 序号 中，有些是字符串，并且包含字符 “*”，需要替换，最后转换为数字
    data['序号'] = data['序号'].astype(str)
    data['序号'] = data['序号'].str.replace('\*','',regex=True)
    data['序号'] = pd.to_numeric(data['序号'])
    
    data = data.sort_values(['季度','持仓市值'],ascending=[True,False])
    df = data.set_index(['序号','季度']).stack().unstack([1,2]).head(10)
    df = df.loc[:,(slice(None), '股票名称')] # 只选取 股票名称
    df = df.droplevel(None,axis=1)
    df.columns.name=None
    df.reset_index(inplace=True)
    """
    df['基金代码']=code
    cols = df.columns.tolist()
    cols = cols[:1] + cols[-1:] + cols[1:-1] # 将基金代码列名放前面
    df = df[cols]
    """
    #设置打印对齐
    pd.set_option('display.max_columns', 1000)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)

    #获取基金名称
    names = ak.fund_em_fund_name()
    namedf=names[names['基金代码']==fund]
    if len(namedf)==0:
        name=fund
    else:
        name=namedf['基金简称'].values[0]
        
    print("=== 中国基金持仓股票排行分析："+name,"(按持仓比例高低排列，近"+str(year_num)+"年前10名) ===\n")
    print(df.to_string(index=False))
    import datetime; today = datetime.date.today()
    print("\n*** 注：包括A股与非A股。数据来源：天天基金/东方财富,",today)          
    
    return df,data

#==============================================================================
if __name__=='__main__':
    fund='180801'
    top=10

def reits_china(fund='',top=10):
    """
    需要升级akshare
    """
    import akshare as ak
    try:
        df1 = ak.reits_info_jsl()
        df2 = ak.reits_realtime_em()
    except:
        print("  #Error (reits_profile_china): you may need to upgrade akshare")
        return None
    
    #合成基金类型信息
    import pandas as pd
    df = pd.merge(df1,df2,on = ['代码'],how='left')    
    df.rename(columns={'涨幅':'涨幅%','成交额_x':'成交额(万元)','折价率':'折价率%','规模':'规模(亿元)','剩余年限':'剩余年限(年)','涨跌幅':'涨跌幅%'}, inplace=True)
    num=len(df)
    
    df.sort_values(by=['昨收'],ascending=False,inplace=True)
    df.reset_index(drop=True,inplace=True)
    import datetime
    today = datetime.date.today()
    
    dfa=df[df['代码']==fund]
    # 未找到
    if len(dfa)==0:
        if top > 0:
            dfa=df.head(top)
        else:
            dfa=df.tail(-top)
        dfb=dfa[['代码','名称','昨收','规模(亿元)','到期日']]
        
        #设置打印对齐
        pd.set_option('display.max_columns', 1000)
        pd.set_option('display.width', 1000)
        pd.set_option('display.max_colwidth', 1000)
        pd.set_option('display.unicode.ambiguous_as_wide', True)
        pd.set_option('display.unicode.east_asian_width', True)
        
        order='前'
        if top <0: 
            order='后'
            top=-top
        print("\n===== 中国REITs基金列表(按收盘价高低排列，"+order+str(top)+"名) =====\n")
        print(dfb)

        print("*** 数据来源：东方财富/集思录, 总计"+str(num)+"只REITs基金,",today)         
        return dfb

    #单列一只基金的具体信息
    collist=['代码','简称','名称','全称','项目类型','基金公司','规模(亿元)','到期日','剩余年限(年)','净值','净值日期','现价','涨幅%','开盘价','最高价','最低价','昨收','成交额(万元)']
    maxcollen=0
    for i in collist:
        ilen=hzlen(i)
        if maxcollen < ilen:
            maxcollen=ilen
            
    dfb=dfa[collist]
    print("\n===== 中国REITs基金详情(代码"+fund+") =====\n")
    for i in collist:
        print(i,' '*(maxcollen-hzlen(i))+'：',dfb[i].values[0])

    print("*** 数据来源：东方财富/集思录,",today)         
    return dfb  
#==============================================================================

if __name__=='__main__':
    fund_type='全部类型'

def pof_list_china(fund_type='全部类型',printout=True):
    """
    功能：抓取公募基金列表，按照基金类型列表，按照基金名称拼音排序
    """
    print("Searching for publicly offering fund (POF) information in China ...")
    import akshare as ak
    
    #基金基本信息：基金代码，基金简称，基金类型
    df = ak.fund_em_fund_name()
    df.sort_values(by=['拼音全称'],na_position='first',inplace=True)
    df.drop_duplicates(subset=['基金代码','基金类型'], keep='first',inplace=True)    
    
    #获取基金类型列表，并去掉重复项
    typelist=list(set(list(df['基金类型'])))
    #判断类型是否支持
    if fund_type not in typelist+['全部类型']:
        print("  #Error(fund_list_china): unsupported fund type:",fund_type)
        print("  Supported fund_type:",typelist+['全部类型'])
        return None

    #摘取选定的基金类型
    if fund_type != '全部类型':
        df2=df[df['基金类型']==fund_type]
    else:
        df2=df
    df3=df2[['基金简称','基金代码','基金类型']]
    df3.reset_index(drop=True,inplace=True) 
    
    #打印种类数量信息    
    if printout:
        num=len(df3)
        if fund_type != '全部类型':
            print(texttranslate("共找到")+str(num)+texttranslate("支基金, 类型为")+fund_type)
            return df3
        
        print("\n",texttranslate("======= 中国公募基金种类概况 ======="))
        print(texttranslate("公募基金总数："),"{:,}".format(num))
        print(texttranslate("其中包括："))
        
        #typelist.sort(reverse=False)
        typelist=['股票型','股票指数','股票-FOF','债券型','债券指数','定开债券', \
                  '混合型','混合-FOF','货币型','ETF-场内','QDII','QDII-指数', \
                  'QDII-ETF','理财型']
        maxlen=0
        for t in typelist:
            tlen=strlen(t)
            if tlen > maxlen: maxlen=tlen
        maxlen=maxlen+1
        
        for t in typelist:
            tlen=strlen(t)
            n=len(df[df['基金类型']==t])
            prefix=' '*4+t+' '*(maxlen-tlen)+':'
            print(prefix,"{:,}".format(n),"\b,",round(n/num*100,2),'\b%')

        import datetime
        today = datetime.date.today()
        print(texttranslate("数据来源：东方财富/天天基金,"),today)
        
    return df3

if __name__=='__main__':
    df=pof_list_china()

#==============================================================================
if __name__=='__main__':
    info_type='单位净值'
    fund_type='债券型'

def oef_rank_china(info_type='单位净值',fund_type='全部类型',rank=10):
    """
    功能：中国开放式基金排名，单位净值，累计净值，手续费
    """
    typelist=['单位净值','累计净值','手续费']
    if info_type not in typelist:
        print("  #Error(oef_rank_china): unsupported info type",info_type)
        print("  Supported info type:",typelist)
        return None
    
    print("Searching for open-ended fund (OEF) information in China ...")
    import akshare as ak   
    
    #获取开放式基金实时信息
    df1 = ak.fund_em_open_fund_daily()
    collist=list(df1)
    nvname1=collist[2]
    nvdate=nvname1[:10]
    nvname2=collist[3]
    #修改列名
    df1.rename(columns={nvname1:'单位净值',nvname2:'累计净值'}, inplace=True) 
    #df1a=df1.drop(df1[df1['单位净值']==''].index)
    #df1b=df1a.drop(df1a[df1a['累计净值']==''].index)
    df1c=df1[['基金代码','基金简称','单位净值','累计净值','申购状态','赎回状态','手续费']]
    
    
    #获取所有公募基金类型信息
    df2 = ak.fund_em_fund_name()
    df2a=df2[['基金代码','基金类型']]
    
    #合成基金类型信息
    import pandas as pd
    df = pd.merge(df1c,df2a,on = ['基金代码'],how='left')
    #过滤基金类型
    if fund_type != '全部类型':
        fundtypelist=list(set(list(df['基金类型'])))
        found=False
        for ft in fundtypelist:
            if fund_type in ft: 
                found=True
                break
        if not found:
            print("  #Error(oef_rank_china): unsupported fund type",fund_type)
            print("  Supported fund type:",fundtypelist)
            return None
        fund_filter=lambda x: fund_type in x
        df['基金类型s']=df['基金类型'].apply(fund_filter)
        df=df[df['基金类型s']==True]    

    
    if info_type == '单位净值':
        df.sort_values(by=['单位净值'],ascending=False,inplace=True)
        dfprint=df[['基金简称','基金代码','基金类型','单位净值','累计净值','手续费']]
        print(texttranslate("\n===== 中国开放式基金排名：单位净值 ====="))
    
    if info_type == '累计净值':
        df.sort_values(by=['累计净值'],ascending=False,inplace=True)
        dfprint=df[['基金简称','基金代码','基金类型','累计净值','单位净值','手续费']]
        print(texttranslate("\n===== 中国开放式基金排名：累计净值 ====="))      
    
    if info_type == '手续费':
        df.sort_values(by=['手续费'],ascending=False,inplace=True)
        dfprint=df[['基金简称','基金代码','基金类型','手续费']]
        print(texttranslate("\n===== 中国开放式基金排名：手续费 ====="))          
        
    #设置打印对齐
    pd.set_option('display.max_columns', 1000)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)
    
    dfprint.dropna(inplace=True)
    dfprint.reset_index(drop=True,inplace=True)
    if rank >= 0:
        dfprint10=dfprint.head(rank)
    else:
        dfprint10=dfprint.tail(-rank)
    #print(dfprint10.to_string(index=False))
    print(dfprint10)
    print(texttranslate("共找到披露净值信息的开放式基金数量:"),len(dfprint),'\b. ',end='')
    print(texttranslate("基金类型:"),fund_type)
    
    print(texttranslate("净值日期:"),nvdate,'\b. ',end='')
    import datetime
    today = datetime.date.today()
    print(texttranslate("数据来源：东方财富/天天基金,"),today)        
    
    return df

if __name__=='__main__':
     df=oef_rank_china(info_type='单位净值')
     df=oef_rank_china(info_type='累计净值')
     df=oef_rank_china(info_type='手续费')

#==============================================================================
if __name__=='__main__':
    fund='050111'
    fromdate='2020-9-1'
    todate='2021-9-1'
    trend_type='排名'
    power=0
    twinx=False
    zeroline=False

def oef_trend_china(fund,fromdate,todate,trend_type='净值',power=0,twinx=False):
    """
    功能：开放式基金业绩趋势，单位净值，累计净值，近三个月收益率，同类排名，总排名
    """
    #检查走势类型
    trendlist=["净值","收益率","排名"]
    if trend_type not in trendlist:
        print("  #Error(oef_trend_china): unsupported trend type:",trend_type)
        print("  Supported trend types:",trendlist)
        return None
    
    #检查日期
    result,start,end=check_period(fromdate,todate)
    if not result:
        print("  #Error(oef_trend_china): invalid date period:",fromdate,todate)
        return None
    """
    #转换日期格式
    import datetime
    startdate=datetime.datetime.strftime(start,"%Y-%m-%d")
    enddate=str(datetime.datetime.strftime(end,"%Y-%m-%d"))
    """
    print("Searching for open-ended fund (OEF) trend info in China ...")
    import akshare as ak   

    #开放式基金-历史数据
    import datetime; today = datetime.date.today()
    source=texttranslate("数据来源：东方财富/天天基金")

    
    #绘制单位/累计净值对比图
    if trend_type == '净值':
        df1 = ak.fund_em_open_fund_info(fund=fund, indicator="单位净值走势")
        df1.rename(columns={'净值日期':'date','单位净值':'单位净值'}, inplace=True)
        df1['日期']=df1['date']
        df1.set_index(['date'],inplace=True) 
        
        df2 = ak.fund_em_open_fund_info(fund=fund, indicator="累计净值走势")
        df2.rename(columns={'净值日期':'date','累计净值':'累计净值'}, inplace=True)
        df2.set_index(['date'],inplace=True)       
        
        #合并
        import pandas as pd
        df = pd.merge(df1,df2,left_index=True,right_index=True,how='inner')
        dfp=df[(df['日期'] >= start)]
        dfp=dfp[(dfp['日期'] <= end)]
        if len(dfp) == 0:
            print("  #Error(oef_trend_china): no info found for",fund,"in the period:",fromdate,todate)
            return
        
        #绘制双线图
        ticker1=fund; colname1='单位净值';label1=texttranslate('单位净值')
        ticker2=fund; colname2='累计净值';label2=texttranslate('累计净值')
        #ylabeltxt='人民币元'
        ylabeltxt=texttranslate('净值')
        titletxt=texttranslate("开放式基金的净值趋势：")+codetranslate(fund)
        
        footnote=source+', '+str(today)
        plot_line2(dfp,ticker1,colname1,label1, \
               dfp,ticker2,colname2,label2, \
               ylabeltxt,titletxt,footnote,power=power,twinx=twinx)
        return df
    
    #绘制累计收益率单线图
    if trend_type == '收益率':
        df = ak.fund_em_open_fund_info(fund=fund, indicator="累计收益率走势")
        df.rename(columns={'净值日期':'date','累计收益率':'累计收益率'}, inplace=True)
        df['日期']=df['date']
        df.set_index(['date'],inplace=True) 
        dfp=df[(df['日期'] >= start)]
        dfp=dfp[(dfp['日期'] <= end)]  
        if len(dfp) == 0:
            print("  #Error(oef_trend_china): no info found for",fund,"in the period:",fromdate,todate)
            return        
    
        colname='累计收益率'; collabel=texttranslate('累计收益率%')
        ylabeltxt=texttranslate('收益率%')
        titletxt=texttranslate("开放式基金的累计收益率趋势：")+codetranslate(fund)
        footnote=source+', '+str(today)
        plot_line(dfp,colname,collabel,ylabeltxt,titletxt,footnote,power=power)    
        return df
    
    #绘制同类排名图：近三个月收益率
    if trend_type == '排名':
        df1 = ak.fund_em_open_fund_info(fund=fund, indicator="同类排名走势")
        df1.rename(columns={'报告日期':'date','同类型排名-每日近三月排名':'同类排名','总排名-每日近三月排名':'总排名'}, inplace=True)
        df1['日期']=df1['date']
        df1['总排名']=df1['总排名'].astype('int64')
        df1.set_index(['date'],inplace=True) 
        
        df2 = ak.fund_em_open_fund_info(fund=fund, indicator="同类排名百分比")
        df2.rename(columns={'报告日期':'date','同类型排名-每日近3月收益排名百分比':'同类排名百分比'}, inplace=True)
        df2.set_index(['date'],inplace=True)       
        
        #合并
        import pandas as pd
        df = pd.merge(df1,df2,left_index=True,right_index=True,how='inner')
        dfp=df[(df['日期'] >= start)]
        dfp=dfp[(dfp['日期'] <= end)]
        if len(dfp) == 0:
            print("  #Error(oef_trend_china): no info found for",fund,"in the period:",fromdate,todate)
            return        

        #绘制双线图：同类排名，总排名
        ylabeltxt=''
        titletxt=texttranslate("开放式基金的近三个月收益率排名趋势：")+codetranslate(fund)
        footnote=source+', '+str(today)        
        
        ticker1=fund; colname1='同类排名';label1=texttranslate('同类排名')
        """
        ticker2=fund; colname2='同类排名百分比';label2=texttranslate('同类排名百分比')
        dfp1=pd.DataFrame(dfp[colname1])
        dfp2=pd.DataFrame(dfp[colname2])
        plot_line2(dfp1,ticker1,colname1,label1, \
               dfp2,ticker2,colname2,label2, \
               ylabeltxt,titletxt,footnote,power=power,twinx=True)
        """
        #    
        ticker2=fund; colname2='总排名';label2=texttranslate('开放式基金总排名')  
        dfp1=pd.DataFrame(dfp[colname1])
        dfp2=pd.DataFrame(dfp[colname2])        
        plot_line2(dfp1,ticker1,colname1,label1, \
               dfp2,ticker2,colname2,label2, \
               ylabeltxt,titletxt,footnote,power=power,twinx=twinx)            
        
        return df
    
#==============================================================================
if __name__=='__main__':
    pass

def mmf_rank_china(rank=10):
    """
    功能：中国货币型基金排名，7日年化收益率%
    """
    
    print("Searching for money market fund (OEF) information in China ...")
    import akshare as ak   
    
    #获取货币型基金实时信息
    df = ak.fund_em_money_fund_daily()
    collist=list(df)
    nvname=collist[6]
    nvdate=nvname[:10]
    #修改列名
    df.rename(columns={nvname:'7日年化%'}, inplace=True) 
    #dfa=df.drop(df[df['7日年化%']==''].index)
    dfb=df[['基金代码','基金简称','7日年化%','成立日期','基金经理','手续费']]
    
    dfb.sort_values(by=['7日年化%'],ascending=False,inplace=True)
    dfprint=dfb[['基金简称','基金代码','7日年化%','基金经理','手续费']]
    print(texttranslate("\n======= 中国货币型基金排名：7日年化收益率 ======="))
        
    #设置打印对齐
    import pandas as pd
    pd.set_option('display.max_columns', 1000)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)
    
    dfprint.dropna(inplace=True)
    dfprint.reset_index(drop=True,inplace=True)
    if rank >=0:
        dfprint10=dfprint.head(rank)
    else:
        dfprint10=dfprint.tail(-rank)
    #print(dfprint10.to_string(index=False))
    print(dfprint10)
    print(texttranslate("共找到披露收益率信息的货币型基金数量:"),len(dfprint))
    
    print(texttranslate("收益率日期:"),nvdate,'\b. ',end='')    
    import datetime
    today = datetime.date.today()
    print(texttranslate("数据来源：东方财富/天天基金,"),today)        
    
    return df

if __name__=='__main__':
     df=mmf_rank_china()

#==============================================================================
if __name__=='__main__':
    fund='320019'
    fromdate='2020-1-1'
    todate='2020-10-16'
    power=0

def mmf_trend_china(fund,fromdate,todate,power=0):
    """
    功能：货币型基金业绩趋势，7日年化收益率
    """
    
    #检查日期
    result,start,end=check_period(fromdate,todate)
    if not result:
        print("  #Error(mmf_trend_china): invalid date period:",fromdate,todate)
        return None
    import datetime
    startdate=datetime.datetime.strftime(start,"%Y-%m-%d")
    enddate=str(datetime.datetime.strftime(end,"%Y-%m-%d"))
    
    print("Searching for money market fund (MMF) trend info in China ...")
    import akshare as ak   

    #基金历史数据
    import datetime; today = datetime.date.today()
    source=texttranslate("数据来源：东方财富/天天基金")

    
    #绘制收益率单线图
    df = ak.fund_em_money_fund_info(fund)
    df.sort_values(by=['净值日期'],ascending=True,inplace=True)
    df['7日年化%']=df['7日年化收益率'].astype("float")
    
    import pandas as pd
    df['date']=pd.to_datetime(df['净值日期'])
    df.set_index(['date'],inplace=True) 
    
    dfp = df[(df.index >= startdate)]
    dfp = dfp[(dfp.index <= enddate)]    
    if len(dfp) == 0:
        print("  #Error(mmf_trend_china): no info found for",fund,"in the period:",fromdate,todate)
        return    
    
    colname='7日年化%'; collabel=texttranslate('7日年化%')
    ylabeltxt=''
    titletxt=texttranslate("货币型基金的7日年化收益率趋势：")+codetranslate(fund)
    footnote=source+', '+str(today)
    plot_line(dfp,colname,collabel,ylabeltxt,titletxt,footnote,power=power)    
    
    return df
    
#==============================================================================
if __name__=='__main__':
    info_type='单位净值'
    fund_type='全部类型'

def etf_rank_china(info_type='单位净值',fund_type='全部类型',rank=10):
    """
    功能：中国ETF基金排名，单位净值，累计净值，手续费
    """
    typelist=['单位净值','累计净值','市价']
    if info_type not in typelist:
        print("  #Error(etf_rank_china): unsupported info type",info_type)
        print("  Supported info type:",typelist)
        return None
    
    print("Searching for exchange traded fund (ETF) information in China ...")
    import akshare as ak   
    
    #获取ETF基金实时信息
    df1 = ak.fund_em_etf_fund_daily()
    #删除全部为空值'---'的列
    df1t=df1.T
    df1t['idx']=df1t.index
    df1t.drop_duplicates(subset=['idx'],keep='last',inplace=True)
    df2=df1t.T
    #删除空值'---'的列
    
    #提取净值日期
    collist=list(df2)
    nvname1=collist[3]
    nvdate=nvname1[:10]
    nvname2=collist[4]
    #修改列名
    df=df2.rename(columns={nvname1:'单位净值',nvname2:'累计净值'}) 
    
    #过滤基金类型
    if fund_type != '全部类型':
        fundtypelist=list(set(list(df['类型'])))
        found=False
        for ft in fundtypelist:
            if fund_type in ft: 
                found=True
                break
        if not found:
            print("  #Error(etf_rank_china): unsupported fund type",fund_type)
            print("  Supported fund type:",fundtypelist)
            return None
        fund_filter=lambda x: fund_type in x
        df['基金类型s']=df['类型'].apply(fund_filter)
        df=df[df['基金类型s']==True]     
    
    if info_type == '单位净值':
        df.sort_values(by=['单位净值'],ascending=False,inplace=True)
        dfprint=df[['基金简称','基金代码','类型','单位净值','累计净值','市价']]
        print(texttranslate("\n===== 中国ETF基金排名：单位净值 ====="))
    
    if info_type == '累计净值':
        df.sort_values(by=['累计净值'],ascending=False,inplace=True)
        dfprint=df[['基金简称','基金代码','类型','累计净值','单位净值','市价']]
        print(texttranslate("\n===== 中国ETF基金排名：累计净值 ====="))        
    
    if info_type == '市价':
        df.sort_values(by=['市价'],ascending=False,inplace=True)
        dfprint=df[['基金简称','基金代码','类型','市价','累计净值','单位净值']]
        print(texttranslate("\n===== 中国ETF基金排名：市价 ====="))          
        
    #设置打印对齐
    import pandas as pd
    pd.set_option('display.max_columns', 1000)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)
    
    dfprint.dropna(inplace=True)
    dfprint.reset_index(drop=True,inplace=True)
    if rank >=0:
        dfprint10=dfprint.head(rank)
    else:
        dfprint10=dfprint.tail(-rank)
    #print(dfprint10.to_string(index=False))
    print(dfprint10)
    print(texttranslate("共找到披露净值信息的ETF基金数量:"),len(dfprint),'\b. ',end='')
    print(texttranslate("基金类型:"),fund_type)
    
    print(texttranslate("净值日期:"),nvdate,'\b. ',end='')
    import datetime
    today = datetime.date.today()
    print(texttranslate("数据来源：东方财富/天天基金,"),today)        
    
    return df

if __name__=='__main__':
     df=etf_rank_china(info_type='单位净值',fund_type='全部类型')
     df=etf_rank_china(info_type='累计净值')
     df=etf_rank_china(info_type='市价')

#==============================================================================
if __name__=='__main__':
    fund='159922'
    fromdate='2020-1-1'
    todate='2020-10-16'

def etf_trend_china(fund,fromdate,todate):
    """
    功能：ETF基金业绩趋势，单位净值，累计净值
    """
    #检查日期
    result,start,end=check_period(fromdate,todate)
    if not result:
        print("  #Error(oef_trend_china): invalid date period:",fromdate,todate)
        return None
    #转换日期格式
    import datetime
    startdate=datetime.datetime.strftime(start,"%Y-%m-%d")
    enddate=str(datetime.datetime.strftime(end,"%Y-%m-%d"))

    print("Searching for exchange traded fund (ETF) trend info in China ...")
    import akshare as ak   

    import datetime; today = datetime.date.today()
    source=texttranslate("数据来源：东方财富/天天基金")

    
    #绘制单位/累计净值对比图
    df = etf_hist_df = ak.fund_em_etf_fund_info(fund)
    import pandas as pd
    df['date']=pd.to_datetime(df['净值日期'])
    df.set_index(['date'],inplace=True) 
    df['单位净值']=df['单位净值'].astype("float")
    df['累计净值']=df['累计净值'].astype("float")
        
    dfp=df[(df['净值日期'] >= startdate)]
    dfp=dfp[(dfp['净值日期'] <= enddate)]
    if len(dfp) == 0:
        print("  #Error(etf_trend_china): no info found for",fund,"in the period:",fromdate,todate)
        return    
        
    #绘制双线图
    ticker1=fund; colname1='单位净值';label1=texttranslate('单位净值')
    ticker2=fund; colname2='累计净值';label2=texttranslate('累计净值')
    ylabeltxt=texttranslate('人民币元')
    titletxt=texttranslate("ETF基金的净值趋势：")+codetranslate(fund)
    footnote=source+', '+str(today)
    
    plot_line2(dfp,ticker1,colname1,label1, \
               dfp,ticker2,colname2,label2, \
               ylabeltxt,titletxt,footnote)
    
    return df
    
if __name__=='__main__':
    df=etf_trend_china('510580','2019-1-1','2020-9-30')
    
#==============================================================================

def fund_summary_china():
    """
    功能：中国基金投资机构概况
    爬虫来源地址：https://zhuanlan.zhihu.com/p/97487003
    """
    print("Searching for fund investment institutions in China ...")
    import akshare as ak

    #会员机构综合查询：
    #机构类型：'商业银行','支付结算机构','证券公司资管子公司','会计师事务所',
    #'保险公司子公司','独立服务机构','证券投资咨询机构','证券公司私募基金子公司',
    #'私募基金管理人','公募基金管理公司','地方自律组织','境外机构','期货公司',
    #'独立第三方销售机构','律师事务所','证券公司','其他机构','公募基金管理公司子公司',
    #'期货公司资管子公司','保险公司'
    amac_member_info_df = ak.amac_member_info()
        
    typelist=['公募基金管理公司','公募基金管理公司子公司','私募基金管理人', \
                '期货公司','期货公司资管子公司','证券公司', \
                '证券公司私募基金子公司','证券公司资管子公司','境外机构']
    maxlen=0
    for t in typelist:
        tlen=strlen(t)
        if tlen > maxlen: maxlen=tlen
    maxlen=maxlen+1
        
    print(texttranslate("\n===== 中国基金投资机构概况 ====="))
    print(texttranslate("机构（会员）数量："),end='')
    num=len(list(set(list(amac_member_info_df["机构（会员）名称"]))))
    print("{:,}".format(num))
        
    print(texttranslate("其中包括："))
    for t in typelist:
        tlen=strlen(t)
        df_sub=amac_member_info_df[amac_member_info_df['机构类型']==t]
        n=len(list(set(list(df_sub['机构（会员）名称']))))
        prefix=' '*4+t+' '*(maxlen-tlen)+':'
        print(prefix,"{:,}".format(n),"\b,",round(n/num*100,2),'\b%')       
        
    import datetime; today = datetime.date.today()
    source=texttranslate("数据来源：中国证券投资基金业协会")
    footnote=source+', '+str(today)  
    print(footnote)
        
    print(texttranslate("\n===== 中国基金投资机构会员代表概况 ====="))
    print(texttranslate("会员代表人数："),end='')
    num=len(list(set(list(amac_member_info_df["会员代表"]))))
    print("{:,}".format(num))        
        
    print(texttranslate("其中工作在："))
    for t in typelist:
        tlen=strlen(t)
        df_sub=amac_member_info_df[amac_member_info_df['机构类型']==t]
        n=len(list(set(list(df_sub['会员代表']))))
        prefix=' '*4+t+' '*(maxlen-tlen)+':'
        print(prefix,"{:,}".format(n),"\b,",round(n/num*100,2),'\b%')  
    print(footnote)        
    
    return amac_member_info_df


#==============================================================================
if __name__=='__main__':
    location='全国'

def pef_manager_china(location='全国'):
    """
    功能：中国私募基金管理人地域分布概况
    爬虫来源地址：https://zhuanlan.zhihu.com/p/97487003
    """
    
    print("Searching for private equity fund (PEF) managers info in China ...")
    import akshare as ak
    import pandas as pd

    #私募基金管理人综合查询
    manager_df = ak.amac_manager_info()
    num=len(list(manager_df["法定代表人/执行事务合伙人(委派代表)姓名"]))
    
    #注册地检查
    if location != '全国':
        typelist=sort_pinyin(list(set(list(manager_df['注册地']))))
        typelist.remove('')
        if location not in typelist:
            print("  #Error(pef_manager_china): failed to find registration place-"+location)
            print("  Supported registration place：",typelist+['全国'])
            return

    #设置打印对齐
    pd.set_option('display.max_columns', 1000)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)

    import datetime; today = datetime.date.today()
    source=texttranslate("数据来源：中国证券投资基金业协会")
    footnote=source+', '+str(today)          
    
    if location != '全国':
        manager_df=manager_df[manager_df['注册地']==location]
        print(texttranslate("\n===== 中国私募基金管理人角色分布 ====="))
        print(texttranslate("地域：")+location)
        print(texttranslate("法定代表人/执行合伙人数量："),end='')
        num1=len(list(manager_df["法定代表人/执行事务合伙人(委派代表)姓名"]))
        print("{:,}".format(num1),texttranslate('\b, 占比全国'),round(num1/num*100.0,2),'\b%')
        
        print(texttranslate("其中, 角色分布："))
        #instlist=list(set(list(manager_df['机构类型'])))
        instlist=['私募股权、创业投资基金管理人','私募证券投资基金管理人','私募资产配置类管理人','其他私募投资基金管理人']
        mtype=pd.DataFrame(columns=['管理人类型','人数','占比%'])
        for t in instlist:
            df_sub=manager_df[manager_df['机构类型']==t]
            n=len(list(df_sub['法定代表人/执行事务合伙人(委派代表)姓名']))
            pct=round(n/num1*100,2)
            s=pd.Series({'管理人类型':t,'人数':n,'占比%':pct})
            mtype=mtype.append(s,ignore_index=True) 
        mtype.sort_values(by=['人数'],ascending=False,inplace=True)
        mtype.reset_index(drop=True,inplace=True)        
        
        print(mtype)
        print(footnote)
        return manager_df
    
    print(texttranslate("\n== 中国私募基金管理人地域分布概况 =="))
    print(texttranslate("法定代表人/执行合伙人数量："),end='')
    num=len(list(manager_df["法定代表人/执行事务合伙人(委派代表)姓名"]))
    print("{:,}".format(num))  
        
    typelist=sort_pinyin(list(set(list(manager_df['注册地']))))
    typelist.remove('')
        
    print(texttranslate("其中分布在："))
    location=pd.DataFrame(columns=['注册地','人数','占比%'])
    for t in typelist:
        df_sub=manager_df[manager_df['注册地']==t]
        n=len(list(df_sub['法定代表人/执行事务合伙人(委派代表)姓名']))
        pct=round(n/num*100,2)
        s=pd.Series({'注册地':t,'人数':n,'占比%':pct})
        location=location.append(s,ignore_index=True) 
    location.sort_values(by=['人数'],ascending=False,inplace=True)
        
    location.reset_index(drop=True,inplace=True)
    location10=location.head(10)
    pctsum=round(location10['占比%'].sum(),2)
    
    print(location10)
    print(texttranslate("上述地区总计占比:"),pctsum,'\b%')
    print(footnote)             
    
    """
    print("\n===== 中国私募基金管理人角色分布 =====")
    print("地域："+location)
    print("法定代表人/执行合伙人数量：",end='')
    num1=len(list(manager_df["法定代表人/执行事务合伙人(委派代表)姓名"]))
    print("{:,}".format(num1),'\b, 占比全国',round(num1/num*100.0,2),'\b%')
        
    print("其中, 角色分布：")
    #instlist=list(set(list(manager_df['机构类型'])))
    instlist=['私募股权、创业投资基金管理人','私募证券投资基金管理人','私募资产配置类管理人','其他私募投资基金管理人']
    mtype=pd.DataFrame(columns=['管理人类型','人数','占比%'])
    for t in instlist:
        df_sub=manager_df[manager_df['机构类型']==t]
        n=len(list(df_sub['法定代表人/执行事务合伙人(委派代表)姓名']))
        pct=round(n/num1*100,2)
        s=pd.Series({'管理人类型':t,'人数':n,'占比%':pct})
        mtype=mtype.append(s,ignore_index=True) 
    mtype.sort_values(by=['人数'],ascending=False,inplace=True)
    mtype.reset_index(drop=True,inplace=True)        
        
    print(mtype)
    print(footnote)
    """
    
    return manager_df


#==============================================================================

def pef_product_china():
    
    """
    功能：中国私募基金管理人的产品管理概况
    爬虫来源地址：https://zhuanlan.zhihu.com/p/97487003
    """
    print("Searching for private equity fund (PEF) products info in China ...")
    import akshare as ak
    import pandas as pd

    #私募基金管理人基金产品
    product_df = ak.amac_fund_info()
    
    print(texttranslate("\n== 中国私募基金管理人的产品与运营概况 =="))
    print(texttranslate("产品数量："),end='')
    num=len(list(product_df["基金名称"]))
    print("{:,}".format(num))  
        
    #管理类型
    print(texttranslate("产品的运营方式分布："))
    #typelist=list(set(list(product_df['私募基金管理人类型'])))
    typelist=['受托管理','顾问管理','自我管理']
    for t in typelist:
            df_sub=product_df[product_df['私募基金管理人类型']==t]
            n=len(list(set(list(df_sub['基金名称']))))
            prefix=' '*4+t+':'
            print(prefix,"{:,}".format(n),"\b,",round(n/num*100,2),'\b%')     
        
    #运行状态
    print(texttranslate("产品的运营状态分布："))
    """
    typelist=list(set(list(product_df['运行状态'])))
    typelist=['状态不明' if i =='' else i for i in typelist]
    """
    typelist=['正在运作','提前清算','正常清算','延期清算','投顾协议已终止','']
    maxlen=0
    for t in typelist:
            tlen=strlen(t)
            if tlen > maxlen: maxlen=tlen
    maxlen=maxlen+1 
        
    for t in typelist:
            df_sub=product_df[product_df['运行状态']==t]
            n=len(list(set(list(df_sub['基金名称']))))
            if t =='': t='状态不明'
            tlen=strlen(t)
            prefix=' '*4+t+' '*(maxlen-tlen)+':'
            print(prefix,"{:,}".format(n),"\b,",round(n/num*100,2),'\b%')      

    import datetime; today = datetime.date.today()
    source=texttranslate("数据来源：中国证券投资基金业协会")
    footnote=source+', '+str(today)       
    print(footnote)
        
    #推出产品数量排行
    print(texttranslate("\n===== 中国推出产品数量最多的私募基金管理人 ====="))
    subttl=pd.DataFrame(product_df.groupby(by=['私募基金管理人名称'])['基金名称'].count())
    subttl.rename(columns={'基金名称':'产品数量'}, inplace=True)
    subttl['占比‰']=round(subttl['产品数量']/num*1000.0,2)
    subttl.sort_values(by=['产品数量'],ascending=False,inplace=True)
    subttl.reset_index(inplace=True)
    subttl10=subttl.head(10)
        
    #设置打印对齐
    pd.set_option('display.max_columns', 1000)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)
    
    print(subttl10)
    
    pctsum=round(subttl10['占比‰'].sum(),2)    
    print(texttranslate("上述产品总计占比:"),pctsum,'\b‰')     
    print(footnote)
        
    print(texttranslate("\n===== 中国私募基金管理人的产品托管概况 ====="))
    #托管产品数量排行
    tnum=len(list(set(list(product_df['托管人名称']))))
    print(texttranslate("托管机构数量:"),"{:,}".format(tnum))
    
    subttl=pd.DataFrame(product_df.groupby(by=['托管人名称'])['基金名称'].count())
    subttl.rename(columns={'基金名称':'产品数量'}, inplace=True)
    subttl.sort_values(by=['产品数量'],ascending=False,inplace=True)
    subttl.reset_index(inplace=True)
        
    subttl=subttl[subttl['托管人名称']!='']
    #subttl.drop(subttl.index[0], inplace=True)       # 删除第1行
    subttl.reset_index(drop=True,inplace=True)
    subttl['占比%']=round(subttl['产品数量']/num*100.0,2)
    subttl10=subttl.head(10)
        
    pctsum=round(subttl10['占比%'].sum(),2)
    print(subttl10)
    print(texttranslate("上述金融机构托管产品总计占比:"),pctsum,'\b%')
    print(footnote)     
        
    return product_df   


#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#以下信息专注于中国内地基金信息，来源于akshare，尚未利用
#==============================================================================
def fund_info_china():
    
    #证券公司集合资管产品
    cam_df = ak.amac_securities_info()
    
    #证券公司直投基金：
    #中国证券投资基金业协会-信息公示-私募基金管理人公示-基金产品公示-证券公司直投基金
    sdif_df = ak.amac_aoin_info()
    
    #证券公司私募投资基金
    speif_df = ak.amac_fund_sub_info()
    
    #证券公司私募基金子公司管理人信息
    spesub_manager_df = ak.amac_member_sub_info()
    
    #基金公司及子公司集合资管产品
    #中国证券投资基金业协会-信息公示-私募基金管理人公示-基金产品公示-基金公司及子公司集合资管产品
    sscam_df = ak.amac_fund_account_info()
    
    #期货公司集合资管产品
    #中国证券投资基金业协会-信息公示-私募基金管理人公示-基金产品公示-期货公司集合资管产品
    fccam_df = ak.amac_futures_info()
    
    #==========================================================================
    #以下为公募数据：
    
    #基金净值估算数据，当前获取在交易日的所有基金的净值估算数据
    #爬虫来源：https://zhuanlan.zhihu.com/p/140478554?from_voters_page=true
    #信息内容：基金代码，基金类型，单位净值，基金名称
    fnve_df = ak.fund_em_value_estimation()
    
    #挑选QDII产品
    fnve_list=list(set(list(fnve_df['基金类型'])))
    qdii=lambda x: True if 'QDII' in x else False
    fnve_df['is_QDII']=fnve_df['基金类型'].apply(qdii)
    fnve_qdii_df=fnve_df[fnve_df['is_QDII']==True]
    
    #基金持股：获取个股的基金持股数据
    #爬虫来源：https://my.oschina.net/akshare/blog/4428824
    #持股的基金类型：symbol="基金持仓"; choice of {"基金持仓", "QFII持仓", "社保持仓", "券商持仓", "保险持仓", "信托持仓"}
    #返回：单次返回指定 symbol 和 date 的所有历史数据
    df = ak.stock_report_fund_hold(symbol="基金持仓", date="20200630")
    
    ###Fama-French三因子回归A股实证（附源码）
    #代码来源：https://mp.weixin.qq.com/s?__biz=MzU5NDY0NDM2NA==&mid=2247486057&idx=1&sn=0fb3f8558da4e55789ce340c03b648cc&chksm=fe7f568ac908df9c22bae8b52207633984ec91ef7b2728eea8c6a75089b8f2db284e3d611775&scene=21#wechat_redirect

    ###Carhart四因子模型A股实证（附源码）
    #代码来源：https://my.oschina.net/akshare/blog/4340998
    
    #==========================================================================
    ###其他公募基金实时/历史行情
    #爬虫来源：https://cloud.tencent.com/developer/article/1624480
    
    ###########XXX理财型基金-实时数据
    #基金代码，基金简称，当前交易日-7日年化收益率，封闭期，申购状态
    wmf_df = ak.fund_em_financial_fund_daily()
    #理财型基金-历史数据
    #净值日期，7日年化收益率，申购状态，赎回状态
    wmf_hist_df = ak.fund_em_financial_fund_info(fund="000134")
    
    ###########分级基金(结构化基金)-实时数据
    #基金代码，基金简称，单位净值，累计净值，市价，折价率，手续费
    gsf_df = ak.fund_em_graded_fund_daily()
    #分级基金-历史数据
    #净值日期，7日年化收益率，申购状态，赎回状态
    gsf_hist_df = ak.fund_em_graded_fund_info(fund="150232")
    
    ###抓取沪深股市所有指数关联的公募基金列表（含ETF、增强、分级等）
    #代码来源：https://blog.csdn.net/leeleilei/article/details/106124894
    
    ###pyecharts绘制可伸缩蜡烛图
    #代码地址：https://segmentfault.com/a/1190000021999451?utm_source=sf-related
    
#==============================================================================






























# -*- coding: utf-8 -*-
"""
本模块功能：投资组合的风险调整收益率教学插件(算法II)
所属工具包：证券投资分析工具SIAT 
SIAT：Security Investment Analysis Tool
创建日期：2018年10月16日
最新修订日期：2020年8月17日
作者：王德宏 (WANG Dehong, Peter)
作者单位：北京外国语大学国际商学院
作者邮件：wdehong2000@163.com
版权所有：王德宏
用途限制：仅限研究与教学使用，不可商用！商用需要额外授权。
特别声明：作者不对使用本工具进行证券投资导致的任何损益负责！
"""

#==============================================================================
#关闭所有警告
import warnings; warnings.filterwarnings('ignore')
from siat.common import *
from siat.translate import *
from siat.security_prices import *
#==============================================================================
import matplotlib.pyplot as plt

#处理绘图汉字乱码问题
import sys; czxt=sys.platform
if czxt in ['win32','win64']:
    plt.rcParams['font.sans-serif'] = ['SimHei']  # 设置默认字体
    mpfrc={'font.family': 'SimHei'}

if czxt in ['darwin']: #MacOSX
    plt.rcParams['font.family']= ['Heiti TC']
    mpfrc={'font.family': 'Heiti TC'}

if czxt in ['linux']: #website Jupyter
    plt.rcParams['font.family']= ['Heiti TC']
    mpfrc={'font.family':'Heiti TC'}

# 解决保存图像时'-'显示为方块的问题
plt.rcParams['axes.unicode_minus'] = False 
#==============================================================================
def get_ff_factors(start,end,scope='US',factor='FF3',freq='daily'):
    
    import pandas as pd
    s=pd.DataFrame([
        ['US','FF3','monthly','F-F_Research_Data_Factors',0],
        ['US','FF3','yearly','F-F_Research_Data_Factors',1],
        ['US','FF3','weekly','F-F_Research_Data_Factors_weekly',0],
        ['US','FF3','daily','F-F_Research_Data_Factors_daily',0],
        ['US','FF5','monthly','F-F_Research_Data_5_Factors_2x3',0],
        ['US','FF5','yearly','F-F_Research_Data_5_Factors_2x3',1],
        ['US','FF5','daily','F-F_Research_Data_5_Factors_2x3_daily',0],  
        ['US','Mom','monthly','F-F_Momentum_Factor',0],
        ['US','Mom','yearly','F-F_Momentum_Factor',1],
        ['US','Mom','daily','F-F_Momentum_Factor_daily',0],  
        ['US','ST_Rev','monthly','F-F_ST_Reversal_Factor',0],
        ['US','ST_Rev','yearly','F-F_ST_Reversal_Factor',1],
        ['US','ST_Rev','daily','F-F_ST_Reversal_Factor_daily',0],    
        ['US','LT_Rev','monthly','F-F_LT_Reversal_Factor',0],
        ['US','LT_Rev','yearly','F-F_LT_Reversal_Factor',1],
        ['US','LT_Rev','daily','F-F_LT_Reversal_Factor_daily',0],   \
        ['Global','FF3','monthly','Global_3_Factors',0],
        ['Global','FF3','yearly','Global_3_Factors',1],
        ['Global','FF3','daily','Global_3_Factors_Daily',0],   
        ['Global_ex_US','FF3','monthly','Global_ex_US_3_Factors',0],
        ['Global_ex_US','FF3','yearly','Global_ex_US_3_Factors',1],
        ['Global_ex_US','FF3','daily','Global_ex_US_3_Factors_Daily',0],  
        ['Europe','FF3','monthly','Europe_3_Factors',0],
        ['Europe','FF3','yearly','Europe_3_Factors',1],
        ['Europe','FF3','daily','Europe_3_Factors_Daily',0],  
        ['Japan','FF3','monthly','Japan_3_Factors',0],
        ['Japan','FF3','yearly','Japan_3_Factors',1],
        ['Japan','FF3','daily','Japan_3_Factors_Daily',0],    
        ['Asia_Pacific_ex_Japan','FF3','monthly','Asia_Pacific_ex_Japan_3_Factors',0],
        ['Asia_Pacific_ex_Japan','FF3','yearly','Asia_Pacific_ex_Japan_3_Factors',1],
        ['Asia_Pacific_ex_Japan','FF3','daily','Asia_Pacific_ex_Japan_3_Factors_Daily',0],   
        ['North_America','FF3','monthly','North_America_3_Factors',0],
        ['North_America','FF3','yearly','North_America_3_Factors',1],
        ['North_America','FF3','daily','North_America_3_Factors_Daily',0], \
        ['Global','FF5','monthly','Global_5_Factors',0],
        ['Global','FF5','yearly','Global_5_Factors',1],
        ['Global','FF5','daily','Global_5_Factors_Daily',0],   
        ['Global_ex_US','FF5','monthly','Global_ex_US_5_Factors',0],
        ['Global_ex_US','FF5','yearly','Global_ex_US_5_Factors',1],
        ['Global_ex_US','FF5','daily','Global_ex_US_5_Factors_Daily',0],  
        ['Europe','FF5','monthly','Europe_5_Factors',0],
        ['Europe','FF5','yearly','Europe_5_Factors',1],
        ['Europe','FF5','daily','Europe_5_Factors_Daily',0],  
        ['Japan','FF5','monthly','Japan_5_Factors',0],
        ['Japan','FF5','yearly','Japan_5_Factors',1],
        ['Japan','FF5','daily','Japan_5_Factors_Daily',0],    
        ['Asia_Pacific_ex_Japan','FF5','monthly','Asia_Pacific_ex_Japan_5_Factors',0],
        ['Asia_Pacific_ex_Japan','FF5','yearly','Asia_Pacific_ex_Japan_5_Factors',1],
        ['Asia_Pacific_ex_Japan','FF5','daily','Asia_Pacific_ex_Japan_5_Factors_Daily',0],   
        ['North_America','FF5','monthly','North_America_5_Factors',0],
        ['North_America','FF5','yearly','North_America_5_Factors',1],
        ['North_America','FF5','daily','North_America_5_Factors_Daily',0], \
        ['Global','Mom','monthly','Global_Mom_Factor',0],
        ['Global','Mom','yearly','Global_Mom_Factor',1],
        ['Global','Mom','daily','Global_Mom_Factor_Daily',0],   
        ['Global_ex_US','Mom','monthly','Global_ex_US_Mom_Factor',0],
        ['Global_ex_US','Mom','yearly','Global_ex_US_Mom_Factor',1],
        ['Global_ex_US','Mom','daily','Global_ex_US_Mom_Factor_Daily',0],  
        ['Europe','Mom','monthly','Europe_Mom_Factor',0],
        ['Europe','Mom','yearly','Europe_Mom_Factor',1],
        ['Europe','Mom','daily','Europe_Mom_Factor_Daily',0],  
        ['Japan','Mom','monthly','Japan_Mom_Factor',0],
        ['Japan','Mom','yearly','Japan_Mom_Factor',1],
        ['Japan','Mom','daily','Japan_Mom_Factor_Daily',0],    
        ['Asia_Pacific_ex_Japan','Mom','monthly','Asia_Pacific_ex_Japan_MOM_Factor',0],
        ['Asia_Pacific_ex_Japan','Mom','yearly','Asia_Pacific_ex_Japan_MOM_Factor',1],
        ['Asia_Pacific_ex_Japan','Mom','daily','Asia_Pacific_ex_Japan_MOM_Factor_Daily',0],   
        ['North_America','Mom','monthly','North_America_Mom_Factor',0],
        ['North_America','Mom','yearly','North_America_Mom_Factor',1],
        ['North_America','Mom','daily','North_America_Mom_Factor_Daily',0]                 
        ], columns=['scope','factor','freq','symbol','seq'])

    #数据源
    source='famafrench'
    if scope == "China": scope="Asia_Pacific_ex_Japan"
    
    #匹配：scope+factor+freq
    ss=s[s['scope'].isin([scope]) & s['factor'].isin([factor]) \
                                  & s['freq'].isin([freq])]  
    #如果未找到匹配的模式，显示信息后返回
    if len(ss)==0:
        print("  #Warning(get_ff_factors): No data item available for",scope,factor,freq)
        return None

    #重新索引，第1行的索引编号为0
    sss=ss.reset_index(drop=True)    
    #取出对应的symbol
    symbol=sss.iloc[0]['symbol']    
    #取出对应的月(0)/年(1)编号
    seq=sss.iloc[0]['seq']

    #抓取数据
    import pandas_datareader.data as web
    try:
        ds = web.DataReader(symbol,source,start,end)
    except:
        print("  #Error(get_ff_factors): Server did not respond")        
        return None
    
    #提取希望的资产定价因子
    factor_df=ds[seq]
    if len(factor_df)==0:
        print("  #Error(get_ff_factors): Server returned empty data for",start,end,scope,factor,freq)
        return None    
    
    return factor_df


if __name__=='__main__':
    ff3_daily=get_ff_factors('2019-05-01','2019-06-30','US','FF3','daily')

#==============================================================================
def get_rf(startdate,enddate,scope='US',freq='daily'):
    """
    功能：按照市场获得无风险收益率，百分比表示
    输入：开始日期，解释日期，市场，频率
    """    

    try:
        rf_df=get_ff_factors(startdate,enddate,scope,'FF3',freq)
    except:
        print("  #Error(get_rf): rf server did not respond.")
        return None    
    if rf_df is None:
        print("  #Warning(get_rf): fetching risk-free return failed")
        return None

    #强制转换索引格式，彻底消除后续并表的潜在隐患
    rf_df['ffdate']=rf_df.index.astype('str')
    import pandas as pd
    rf_df['ffdate']=pd.to_datetime(rf_df['ffdate'])
    rf_df.set_index(['ffdate'],inplace=True)    
    
    return rf_df

#==============================================================================
#==============================================================================
def calc_treynor_ratio(regdf):
    """
    功能：计算一项特雷诺指数
    输入：数据框，至少含有Ret-Rf和Mkt-Rf两项
    输出：特雷诺指数，Ret-Rf均值
    """
    
    #计算风险溢价Ret-RF均值
    ret_rf_mean=regdf['Ret-RF'].mean()
    
    #使用CAPM回归计算投资组合的贝塔系数，这里得到的alpha就是Jensen's alpha
    from scipy import stats
    output=stats.linregress(regdf['Mkt-RF'],regdf['Ret-RF'])
    (beta,alpha,r_value,p_value,std_err)=output 
    
    #计算特雷诺指数
    tr=ret_rf_mean/beta
    
    ret_mean=regdf['Close'].mean()
    return tr,ret_mean

#==============================================================================
def calc_alpha_ratio(regdf):
    """
    功能：计算一项詹森阿尔法指数
    输入：数据框，至少含有Ret-Rf和Mkt-Rf两项
    输出：詹森阿尔法指数，Ret-Rf均值
    """
    """
    #计算风险溢价Ret-RF均值
    ret_rf_mean=regdf['Ret-RF'].mean()
    """
    #使用CAPM回归计算投资组合的贝塔系数，这里得到的alpha就是Jensen's alpha
    from scipy import stats
    output=stats.linregress(regdf['Mkt-RF'],regdf['Ret-RF'])
    (beta,alpha,r_value,p_value,std_err)=output 
    
    ret_mean=regdf['Close'].mean()
    return alpha,ret_mean

#==============================================================================
def calc_sharpe_ratio(regdf):
    """
    功能：计算一项夏普指数
    输入：数据框，至少含有Ret-Rf和Mkt-Rf两项
    输出：夏普指数，Ret-Rf均值
    """
    #计算风险溢价Ret-RF均值和标准差
    ret_rf_mean=regdf['Ret-RF'].mean()
    ret_rf_std=regdf['Ret-RF'].std()
    
    #计算夏普指数
    sr=ret_rf_mean/ret_rf_std
    
    ret_mean=regdf['Close'].mean()
    return sr,ret_mean


#==============================================================================
def calc_sortino_ratio(regdf):
    """
    功能：计算一项索替诺指数
    输入：数据框，至少含有Ret-Rf和Mkt-Rf两项
    输出：索替诺指数，Ret-Rf均值
    """
    
    #计算风险溢价Ret-RF均值和下偏标准差LPSD
    ret_rf_mean=regdf['Ret-RF'].mean()
    reg2=regdf[regdf['Ret-RF'] < 0]
    ret_rf_lpsd=reg2['Ret-RF'].std()
    
    #计算索梯诺指数
    sr=ret_rf_mean/ret_rf_lpsd
    
    ret_mean=regdf['Close'].mean()
    return sr,ret_mean

#==============================================================================
def print_rar_ratio(regdf,portfolio,ret_mean,ratio_name,ratio):
    """
    功能：打印风险调整后的收益率
    输入：数据框，投资组合构成，收益溢价均值，指数名称，指数
    输出：打印
    """

    #从字典中提取信息
    scope,mktidx,stocklist,portionlist=decompose_portfolio(portfolio)
    stocklist1,_=cvt_yftickerlist(stocklist)
    
    date_start=str(regdf.index[0].year)+'-'+str(regdf.index[0].month)+ \
            '-'+str(regdf.index[0].day)
    date_end=str(regdf.index[-1].year)+'-'+str(regdf.index[-1].month)+ \
            '-'+str(regdf.index[-1].day)            
    print("\n===== 风险调整收益率 =====")
    print("市场指数:",scope,'\b,',codetranslate(mktidx))
    print("成分股  :",stocklist1)
    print("持仓权重:",portionlist)
    print("计算期间:",date_start,"至",date_end)
    print("日均收益率:",round(ret_mean,4),'\b%')
    #print(ratio_name.capitalize(),"\b比率:",round(ratio,4),'\b%')
    print(ratio_name.capitalize(),"\b比率:",round(ratio,4))
    
    import datetime as dt; today=dt.date.today()
    print("*数据来源：雅虎财经, "+str(today))    
    
    return 
#==============================================================================
def treynor_ratio_portfolio(portfolio,start,end,printout=True):
    """
    功能：按天计算一个投资组合的特雷诺指数
    投资组合的结构：{'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    输入：投资组合，开始日期，结束日期
    输出：特雷诺指数
    """

    #仅为测试用
    #portfolio={'Market':('US','^GSPC'),'EDU':0.4,'TAL':0.3,'TEDU':0.2,'RYB':0.1}
    #start='2019-06-01'
    #end  ='2019-06-30'
    #scope='US'
    
    #第1步：各种准备和检查工作
    #设定错误信息的函数名
    func_name='treynor_ratio_portfolio'
    #设定需要计算的指数名称
    ratio_name="treynor"
    result,startdate,enddate=check_period(start,end)
    if not result:
        message="Error #1 ("+func_name+"): "+"Wrong start or end date"
        print(message)
        return None,None    
    
    #从字典中提取信息
    scope,mktidx,stocklist,portionlist=decompose_portfolio(portfolio)
    
    #第2步：计算投资组合的日收益率序列
    #抓取日投资组合价格
    sp=get_portfolio_prices(portfolio,startdate,enddate)
    #计算日收益率，表示为百分比
    import pandas as pd
    ret_pf=pd.DataFrame(sp['Close'].pct_change())*100.0
    ret_pf=ret_pf.dropna()

    #第3步：获得无风险收益率/市场收益率序列
    #获得期间的日无风险收益率(抓取的RF为百分比) 
    rf_df=get_rf(startdate,enddate,scope='US',freq='daily')  
    if rf_df is None:
        message="Error #2 ("+func_name+"): "+"No data available for rf"
        print(message)
        return None,None 
    
    #第4步：合并投资组合日收益率与无风险利率/市场收益率序列
    #合并rf_df与ret_pf
    reg=pd.merge(ret_pf,rf_df,how='inner',left_index=True,right_index=True)
    reg['Ret-RF']=reg['Close']-reg['RF']
    reg=reg.dropna()
    if len(reg) == 0:
        message="Error #2 ("+func_name+"): "+"Empty data for calculation"
        print(message)
        return None,None 
    
    #第5步：计算风险调整后的收益率
    ##########风险调整后的收益率，计算开始##########
    tr,ret_mean=calc_treynor_ratio(reg)
    ##########风险调整后的收益率，计算结束##########
    
    #第6步：打印结果
    if printout == True:
        print_rar_ratio(reg,portfolio,ret_mean,ratio_name,tr)
    
    return tr,ret_mean


if __name__=='__main__':
    portfolio1={'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    tr1,ret1=treynor_ratio_portfolio(portfolio1,'2019-01-01','2019-01-31')


#==============================================================================
def rar_ratio_portfolio(portfolio,start,end,ratio_name='treynor',printout=True):
    """
    功能：按天计算一个投资组合的风险调整后的收益率指数
    投资组合的结构：{'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    输入：投资组合，开始日期，结束日期
    输出：风险调整后的收益率指数
    """

    #仅为测试用
    #portfolio={'Market':('US','^GSPC'),'EDU':0.4,'TAL':0.3,'TEDU':0.2,'RYB':0.1}
    #start='2019-06-01'
    #end  ='2019-06-30'
    #scope='US'
    
    #第1步：各种准备和检查工作
    #设定错误信息的函数名
    func_name='rar_ratio_portfolio'
    
    ratio_list=['treynor','sharpe','sortino','alpha']
    if ratio_name not in ratio_list:
        message="#Error("+func_name+"): "+"Not supported rar ratio type"
        print(message)
        return None,None         
    result,startdate,enddate=check_period(start,end)
    if not result:
        message="#Error("+func_name+"): "+"Wrong start or end date"
        print(message)
        return None,None    
    
    #从字典中提取信息
    scope,mktidx,stocklist,portionlist=decompose_portfolio(portfolio)
    
    #第2步：计算投资组合的日收益率序列
    #抓取日投资组合价格
    sp=get_portfolio_prices(portfolio,startdate,enddate)
    #计算日收益率，表示为百分比
    import pandas as pd
    ret_pf=pd.DataFrame(sp['Close'].pct_change())*100.0
    ret_pf=ret_pf.dropna()

    #第3步：获得无风险收益率/市场收益率序列
    #获得期间的日无风险收益率(抓取的RF为百分比) 
    rf_df=get_rf(startdate,enddate,scope='US',freq='daily')  
    if rf_df is None:
        message="#Error("+func_name+"): "+"No data available for rf"
        print(message)
        return None,None 
    
    #第4步：合并投资组合日收益率与无风险利率/市场收益率序列
    #合并rf_df与ret_pf
    reg=pd.merge(ret_pf,rf_df,how='inner',left_index=True,right_index=True)
    reg['Ret-RF']=reg['Close']-reg['RF']
    reg=reg.dropna()
    if len(reg) == 0:
        message="#Error("+func_name+"): "+"Empty data for ratio calculation"
        print(message)
        return None,None 
    
    #第5步：计算风险调整后的收益率
    ##########风险调整后的收益率，计算开始##########
    calc_func='calc_'+ratio_name+'_ratio'
    rar,ret_mean=eval(calc_func)(reg)
    ##########风险调整后的收益率，计算结束##########
    
    #第6步：打印结果
    if printout == True:
        print_rar_ratio(reg,portfolio,ret_mean,ratio_name,rar)
    
    return rar,ret_mean


if __name__=='__main__':
    pf1={'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    tr1,rp1=rar_ratio_portfolio(pf1,'2019-01-01','2019-01-31',ratio_name='treynor')

#==============================================================================
if __name__=='__main__':
    portfolio={'Market':('US','^GSPC'),'AAPL':1}
    start='2019-12-1'
    end  ='2021-1-31'
    scope='US'
    ratio_name='sharpe'
    window=30
    graph=True    
    
def rar_ratio_rolling(portfolio,start,end,ratio_name='treynor', \
                      window=30,graph=True):
    """
    功能：滚动计算一个投资组合的风险调整后的收益率指数
    投资组合的结构：{'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    输入：投资组合，开始日期，结束日期，指数名称，滚动窗口宽度(天数)
    输出：风险调整后的收益率指数序列
    注意：因需要滚动计算，开始和结束日期之间需要拉开距离，提前的月数为window/21取整+1；
    另外，无风率可用数据可能距离当前日期滞后约两个月
    """
    
    #第1步：各种准备和检查工作
    print("  Start to calculate rar ratios, please wait ...")
    #设定错误信息的函数名
    func_name='rar_ratio_portfolio'
    
    ratio_list=['treynor','sharpe','sortino','alpha']
    if ratio_name not in ratio_list:
        message="#Error("+func_name+"): "+"Not supported rar ratio type"
        print(message)
        return None        
    result,startdate,enddate=check_period(start,end)
    if not result:
        message="  #Error("+func_name+"): "+"Wrong start or end date"
        print(message)
        return None    
    #估算数据提前量，重设开始日历日期
    startdate_delta=int(window/2*3)+30
    startdate1=date_adjust(startdate, adjust=-startdate_delta)
    
    #从字典中提取信息
    scope,mktidx,stocklist,portionlist=decompose_portfolio(portfolio)
    
    #第2步：计算投资组合的日收益率序列
    #抓取日投资组合价格
    sp=get_portfolio_prices(portfolio,startdate1,enddate)
    if sp is None: 
        message="  #Error("+func_name+"): "+"failed to retrieve portfolio info, pls try later"
        print(message)        
        return None
    #计算日收益率，表示为百分比
    import pandas as pd
    ret_pf=pd.DataFrame(sp['Close'].pct_change())*100.0
    ret_pf=ret_pf.dropna()

    #第3步：获得无风险收益率/市场收益率序列
    #获得期间的日无风险收益率(抓取的RF为百分比) 
    rf_df=get_rf(startdate1,enddate,scope=scope,freq='daily')  
    if rf_df is None:
        message="  #Warning("+func_name+"): "+"rf data not available for "+scope
        print(message)
        #以美国的rf数据替代
        rf_df=get_rf(startdate1,enddate,scope='US',freq='daily')
    
    #第4步：合并投资组合日收益率与无风险利率/市场收益率序列
    #合并rf_df与ret_pf
    reg=pd.merge(ret_pf,rf_df,how='inner',left_index=True,right_index=True)
    if len(reg) == 0:
        message="  #Error("+func_name+"): "+"Empty data for ratio calculation"
        print(message)
        return None     
    reg['Ret-RF']=reg['Close']-reg['RF']
    reg=reg.dropna()
    
    #第5步：滚动计算风险调整后的收益率
    ##########风险调整后的收益率，计算开始##########
    #用于保存rar和ret_rf_mean
    import pandas as pd
    import numpy as np
    datelist=reg.index.to_list()
    calc_func='calc_'+ratio_name+'_ratio'
    
    rars=pd.DataFrame(columns=('Date','RAR','Mean(Ret)')) 
    for i in np.arange(0,len(reg)):
        i1=i+window-1
        if i1 >= len(reg): break
        
        #构造滚动窗口
        windf=reg[reg.index >= datelist[i]]
        windf=windf[windf.index <= datelist[i1]]
        #print(i,datelist[i],i1,datelist[i1],len(windf))
        
        #使用滚动窗口计算
        rar,ret_mean=eval(calc_func)(windf)
        
        #记录计算结果
        row=pd.Series({'Date':datelist[i1],'RAR':rar,'Mean(Ret)':ret_mean})
        rars=rars.append(row,ignore_index=True)        
    
    rars.set_index(['Date'],inplace=True) 
    ##########风险调整后的收益率，计算结束##########
    
    #第6步：绘图
    print("  Rendering graphics ...")
    if graph == True:
        draw_rar_ratio(rars,portfolio,ratio_name)
    
    return rars


if __name__=='__main__':
    pf1={'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    rars1=rar_ratio_rolling(pf1,'2020-1-1','2020-12-31',ratio_name='sharpe')
#==============================================================================
def draw_rar_ratio(rars,portfolio,ratio_name):
    """
    功能：绘制滚动窗口曲线
    输入：滚动数据df，投资组合，指数名称
    """
    
    scope,mktidx,stocklist,portionlist=decompose_portfolio(portfolio)
    stocklist1,_=cvt_yftickerlist(stocklist)
    
    """
    #平滑处理
    rars1=rars.resample('H')
    #rars2=rars1.interpolate(method='pchip')
    #rars2=rars1.interpolate(method='akima')
    rars2=rars1.interpolate(method='cubic')
    """
        
    #plt.figure(figsize=(8,5))
 
    labeltxt=ratio_name.capitalize()+'指数'    
    plt.plot(rars['RAR'],label=labeltxt,color='red',lw=1)
    #plt.plot(rars['Mean(Ret)'],label='Stock(s) return(%)',color='blue',lw=1)
    plt.axhline(y=0.0,color='black',linestyle=':')
    
    titletxt='风险调整收益的滚动趋势'+'\n（'+codetranslate(stocklist1)+'）'
    if len(stocklist) > 1:
        titletxt=titletxt+'\n持仓比例: '+str(portionlist)   
    #plt.title(titletxt,fontsize=12,fontweight='bold') 
    plt.title(titletxt,fontsize=12) 
    
    #ylabeltxt="比率/指数"
    #plt.ylabel(ylabeltxt,fontsize=12)
    #plt.xticks(rotation=45,fontsize=9)
    plt.xticks(rotation=30,fontsize=8)
    plt.legend(loc='best') 
    
    import datetime as dt; today=dt.date.today() 
    footnote="数据来源：新浪财经/雅虎财经，"+str(today)
    plt.xlabel(footnote)
    plt.show()

    #使用seaborn绘图
    """
    import seaborn as sns
    with sns.axes_style("whitegrid"):
        fig, ax = plt.subplots(figsize=(8,5))
        ax.plot(rars['RAR'],label=labeltxt,color='red',lw=3)
        #ax.plot(rars['Mean(Ret)'],label='Stock(s) return(%)',color='blue',lw=1)
        plt.axhline(y=0.0,label='Zero return',color='black',linestyle=':')
        ax.set_title(titletxt)
        #ax.set_ylabel(ylabeltxt)
        plt.xticks(rotation=45)
        ax.legend(loc='best')
        ax.set_ylim([1.2*(rars['RAR'].min()), 1.1*(rars['RAR'].max())])    
    """
    return
#==============================================================================
def sharpe_ratio_portfolio(portfolio,start,end,printout=True):
    """
    功能：按天计算一个投资组合的夏普指数
    投资组合的结构：{'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    输入：投资组合，开始日期，结束日期
    输出：夏普指数
    """

    #仅为测试用
    #portfolio={'Market':('US','^GSPC'),'AAPL':0.2,'MSFT':0.6,'IBM':0.2}
    #start='2019-01-01'
    #end  ='2019-01-31'
    
    freq='daily'
    keylist=list(portfolio.keys())
    scope=portfolio[keylist[0]][0]
    mktidx=portfolio[keylist[0]][1]
    
    #从字典中提取信息
    slist=[]
    plist=[]
    for key,value in portfolio.items():
        slist=slist+[key]
        plist=plist+[value]
    stocklist=slist[1:]    
    portionlist=plist[1:]
    
    #检查份额配比是否合理
    """
    if round(sum(portionlist),1) != 1.0:
        print("  #Error(sharpe_ratio): Incorrect total of portions")
        return None,None
    """
    
    #抓取指数和股票价格
    tickerlist=[mktidx]+stocklist
    import pandas as pd
    try:
        startdate=pd.to_datetime(start)
        enddate=pd.to_datetime(end)
    except:
        print("  #Error(sharpe_ratio): incorrect start/end date(s)")
        return None,None       
    
    import pandas_datareader as pdr
    try:
        price=pdr.DataReader(tickerlist,'yahoo',start=startdate,end=enddate)
    except:
        print("  #Error(sharpe_ratio): Server did not respond")
        return None,None
    
    if len(price) == 0:
        print("  #Error(sharpe_ratio): Server returned empty data")
        return None,None
        
    #提取股指和投资组合中各个股票的收盘价    
    closeprice=price['Close'] 

    #将股价乘以各自的份额，放入新的表sp中  
    sp=pd.DataFrame(columns=stocklist)
    for stock in stocklist:
        pos=stocklist.index(stock)
        portion=portionlist[pos]
        sp[stock]=closeprice[stock]*portion

    #逐行汇总汇总，形成每日投资组合的价格
    #在同一行中对各个列的数值计算：axis=1
    sp['PORTFOLIO']=sp.apply(lambda row: sum(row), axis =1)
    ret_pf=pd.DataFrame(sp['PORTFOLIO'].pct_change())*100.0
    ret_pf=ret_pf.dropna()
    
    #获得期间的无风险收益率 
    try:
        rf_df=get_ff_factors(startdate,enddate,scope,'FF3',freq)
    except:
        print("  #Error(sharpe_ratio): RF server did not respond")
        return None,None        
    if len(rf_df) == 0:
        print("  #Error(sharpe_ratio): RF server returned empty data")
        return None,None

    #强制转换索引格式，彻底消除下面并表的潜在隐患
    rf_df['ffdate']=rf_df.index.astype('str')
    rf_df['ffdate']=pd.to_datetime(rf_df['ffdate'])
    rf_df.set_index(['ffdate'],inplace=True)
    
    #合并rf_df与ret_pf
    reg=pd.merge(ret_pf,rf_df,how='inner',left_index=True,right_index=True)
    reg['Ret-RF']=reg['PORTFOLIO']-reg['RF']
    reg=reg.dropna()
    
    #计算风险溢价Ret-RF均值和标准差
    ret_rf_mean=reg['Ret-RF'].mean()
    ret_rf_std=reg['Ret-RF'].std()
    
    #计算夏普指数
    sr=ret_rf_mean/ret_rf_std
    
    #打印报告
    if printout == True:
        date_start=str(reg.index[0].year)+'-'+str(reg.index[0].month)+ \
            '-'+str(reg.index[0].day)
        date_end=str(reg.index[-1].year)+'-'+str(reg.index[-1].month)+ \
            '-'+str(reg.index[-1].day)            
        print("\n===== 风险调整收益率 =====")
        print("投资组合：",portfolio)
        print("计算期间：",date_start,"至",date_end, \
          "(可用日期)")
        print("风险溢价均值(%)：",round(ret_rf_mean,4))
        print("夏普比率(%)：",round(sr,4))
        import datetime as dt; today=dt.date.today()
        print("*数据来源：雅虎财经，"+str(today))
    
    return sr,ret_rf_mean


if __name__=='__main__':
    portfolio={'Market':('US','^GSPC'),'AAPL':0.2,'MSFT':0.5,'IBM':0.3}
    sr1,rp1=sharpe_ratio_portfolio(portfolio,'2019-01-01','2019-01-31')


#==============================================================================
def sortino_ratio_portfolio(portfolio,start,end,printout=True):
    """
    功能：按天计算一个投资组合的索梯诺指数
    投资组合的结构：{'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    输入：投资组合，开始日期，结束日期
    输出：索梯诺指数
    """

    #仅为测试用
    #portfolio={'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    #start='2019-01-01'
    #end  ='2019-01-31'
    
    freq='daily'
    keylist=list(portfolio.keys())
    scope=portfolio[keylist[0]][0]
    mktidx=portfolio[keylist[0]][1]
    
    #从字典中提取信息
    slist=[]
    plist=[]
    for key,value in portfolio.items():
        slist=slist+[key]
        plist=plist+[value]
    stocklist=slist[1:]    
    portionlist=plist[1:]
    
    #检查份额配比是否合理
    """
    if round(sum(portionlist),1) != 1.0:
        print("  #Error(sortino_ratio): Incorrect total of portions")
        return None,None
    """
    
    #抓取指数和股票价格
    tickerlist=[mktidx]+stocklist
    import pandas as pd
    try:
        startdate=pd.to_datetime(start)
        enddate=pd.to_datetime(end)
    except:
        print("  #Error(sortino_ratio): Incorrect start/end date(s)")
        return None,None
        
    import pandas_datareader as pdr
    try:
        price=pdr.DataReader(tickerlist,'yahoo',start=startdate,end=enddate)
    except:
        print("  #Error(sortino_ratio): Server did not respond")
        return None,None
    
    if len(price) == 0:
        print("  #Error(sortino_ratio): Server returned empty data")
        return None,None
        
    #提取股指和投资组合中各个股票的收盘价    
    closeprice=price['Close'] 

    #将股价乘以各自的份额，放入新的表sp中  
    sp=pd.DataFrame(columns=stocklist)
    for stock in stocklist:
        pos=stocklist.index(stock)
        portion=portionlist[pos]
        sp[stock]=closeprice[stock]*portion

    #逐行汇总汇总，形成每日投资组合的价格
    #在同一行中对各个列的数值计算：axis=1
    sp['PORTFOLIO']=sp.apply(lambda row: sum(row), axis =1)
    ret_pf=pd.DataFrame(sp['PORTFOLIO'].pct_change())*100.0
    ret_pf=ret_pf.dropna()
    
    #获得期间的无风险收益率  
    try:      
        rf_df=get_ff_factors(startdate,enddate,scope,'FF3',freq)
    except:
        print("  #Error(sharpe_ratio): RF server did not respond")
        return None,None        

    if rf_df is None:
        print("  #Error(sortino_ratio): RF server did not respond")
        return None,None        
    if len(rf_df) == 0:
        print("Error #7 (sortino_ratio): RF server returned empty data")
        return None,None 
    
    #强制转换索引格式，彻底消除下面并表的潜在隐患
    rf_df['ffdate']=rf_df.index.astype('str')
    rf_df['ffdate']=pd.to_datetime(rf_df['ffdate'])
    rf_df.set_index(['ffdate'],inplace=True)
    
    #合并rf_df与ret_pf
    reg=pd.merge(ret_pf,rf_df,how='inner',left_index=True,right_index=True)
    reg['Ret-RF']=reg['PORTFOLIO']-reg['RF']
    reg=reg.dropna()
    
    #计算风险溢价Ret-RF均值和下偏标准差LPSD
    ret_rf_mean=reg['Ret-RF'].mean()
    reg2=reg[reg['Ret-RF'] < 0]
    ret_rf_lpsd=reg2['Ret-RF'].std()
    
    #计算索梯诺指数
    sr=ret_rf_mean/ret_rf_lpsd
    
    #打印报告
    if printout == True:    
        date_start=str(reg.index[0].year)+'-'+str(reg.index[0].month)+ \
            '-'+str(reg.index[0].day)
        date_end=str(reg.index[-1].year)+'-'+str(reg.index[-1].month)+ \
            '-'+str(reg.index[-1].day)            
        print("\n===== 风险调整收益率 =====")
        print("投资组合：",portfolio)
        print("计算期间：",date_start,"至",date_end, \
              "(可用日期)")
        print("风险溢价均值(%)：",round(ret_rf_mean,4))
        print("索替诺比率(%)：",round(sr,4))
        
        import datetime as dt; today=dt.date.today()
        print("*数据来源：雅虎财经，"+str(today))
    
    return sr,ret_rf_mean


if __name__=='__main__':
    portfolio={'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    sr1,rp1=sortino_ratio_portfolio(portfolio,'2019-01-01','2019-08-03')

#==============================================================================
def jensen_alpha_portfolio(portfolio,start,end,printout=True):
    """
    功能：按天计算一个投资组合的阿尔法指数
    投资组合的结构：{'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    输入：投资组合，开始日期，结束日期
    输出：阿尔法指数
    """

    #仅为测试用
    #portfolio={'Market':('China','000001.SS'),'600519.SS':1.0}
    #start='2019-01-01'
    #end  ='2019-01-31'
    
    freq='daily'
    keylist=list(portfolio.keys())
    scope=portfolio[keylist[0]][0]
    mktidx=portfolio[keylist[0]][1]
    
    #从字典中提取信息
    slist=[]
    plist=[]
    for key,value in portfolio.items():
        slist=slist+[key]
        plist=plist+[value]
    stocklist=slist[1:]    
    portionlist=plist[1:]
    
    #检查份额配比是否合理
    """
    if round(sum(portionlist),1) != 1.0:
        print("  #Error(jensen_alpha): Incorrect total of portions.")
        return None,None
    """
    
    #抓取指数和股票价格
    tickerlist=[mktidx]+stocklist
    import pandas as pd
    try:
        startdate=pd.to_datetime(start)
        enddate=pd.to_datetime(end)
    except:
        print("  #Error(jensen_alpha): Incorrect start/end date(s).")
        return None,None        
        
    import pandas_datareader as pdr
    try:
        price=pdr.DataReader(tickerlist,'yahoo',start=startdate,end=enddate)
    except:
        print("  #Error(jensen_alpha): Server did not respond.")
        return None,None
    
    if len(price) == 0:
        print("  #Error(jensen_alpha): Server returned empty data.")
        return None,None
        
    #提取股指和投资组合中各个股票的收盘价    
    closeprice=price['Close'] 

    #将股价乘以各自的份额，放入新的表sp中  
    sp=pd.DataFrame(columns=stocklist)
    for stock in stocklist:
        pos=stocklist.index(stock)
        portion=portionlist[pos]
        sp[stock]=closeprice[stock]*portion

    #逐行汇总汇总，形成每日投资组合的价格
    #在同一行中对各个列的数值计算：axis=1
    sp['PORTFOLIO']=sp.apply(lambda row: sum(row), axis =1)
    ret_pf=pd.DataFrame(sp['PORTFOLIO'].pct_change())*100.0
    ret_pf=ret_pf.dropna()
    
    #获得期间的无风险收益率        
    rf_df=get_ff_factors(startdate,enddate,scope,'FF3',freq)
    #强制转换索引格式，彻底消除下面并表的潜在隐患
    rf_df['ffdate']=rf_df.index.astype('str')
    rf_df['ffdate']=pd.to_datetime(rf_df['ffdate'])
    rf_df.set_index(['ffdate'],inplace=True)
    
    if rf_df is None:
        print("  #Error(jensen_alpha): RF server did not respond.")
        return None,None        
    if len(rf_df) == 0:
        print("  #Error(jensen_alpha): RF server returned empty data.")
        return None,None 
    
    #合并rf_df与ret_pf
    reg=pd.merge(ret_pf,rf_df,how='inner',left_index=True,right_index=True)
    reg['Ret-RF']=reg['PORTFOLIO']-reg['RF']
    reg=reg.dropna()
    if len(reg) == 0:
        print("  #Error(jensen_alpha): Empty data for regression.")
        return None,None     
    ret_rf_mean=reg['Ret-RF'].mean()
    
    #使用CAPM回归计算投资组合的贝塔系数，这里得到的alpha就是Jensen's alpha
    from scipy import stats
    output=stats.linregress(reg['Mkt-RF'],reg['Ret-RF'])
    (beta,alpha,r_value,p_value,std_err)=output 
    
    #打印报告
    if printout == True:
        date_start=str(reg.index[0].year)+'-'+str(reg.index[0].month)+ \
            '-'+str(reg.index[0].day)
        date_end=str(reg.index[-1].year)+'-'+str(reg.index[-1].month)+ \
            '-'+str(reg.index[-1].day)            
        print("\n===== 风险调整收益率 =====")
        print("投资组合：",portfolio)
        print("计算期间：",date_start,"至",date_end, \
              "(可用日期)")
        print("风险溢价均值(%)：",round(ret_rf_mean,4))
        print("詹森阿尔法(%)：",round(alpha,4))
        
        import datetime as dt; today=dt.date.today()
        print("*数据来源：雅虎财经，"+str(today))        
    
    return alpha,ret_rf_mean


if __name__=='__main__':
    portfolio={'Market':('US','^GSPC'),'AAPL':0.5,'MSFT':0.3,'IBM':0.2}
    alpha1=jensen_alpha_portfolio(portfolio,'2019-01-01','2019-08-03')

#==============================================================================
def calc_monthly_date_range(start,end):
    """
    功能：返回两个日期之间各个月份的开始和结束日期
    输入：开始/结束日期
    输出：两个日期之间各个月份的开始和结束日期元组对列表
    """
    #测试用
    #start='2019-01-05'
    #end='2019-06-25'    
    
    import pandas as pd
    startdate=pd.to_datetime(start)
    enddate=pd.to_datetime(end)

    mdlist=[]
    #当月的结束日期
    syear=startdate.year
    smonth=startdate.month
    import calendar
    sdays=calendar.monthrange(syear,smonth)[1]
    from datetime import date
    slastday=pd.to_datetime(date(syear,smonth,sdays))

    if slastday > enddate: slastday=enddate
    
    #加入第一月的开始和结束日期
    import bisect
    bisect.insort(mdlist,(startdate,slastday))
    
    #加入结束月的开始和结束日期
    eyear=enddate.year
    emonth=enddate.month
    efirstday=pd.to_datetime(date(eyear,emonth,1))   
    if startdate < efirstday:
        bisect.insort(mdlist,(efirstday,enddate))
    
    #加入期间内各个月份的开始和结束日期
    from dateutil.relativedelta import relativedelta
    next=startdate+relativedelta(months=+1)
    while next < efirstday:
        nyear=next.year
        nmonth=next.month
        nextstart=pd.to_datetime(date(nyear,nmonth,1))
        ndays=calendar.monthrange(nyear,nmonth)[1]
        nextend=pd.to_datetime(date(nyear,nmonth,ndays))
        bisect.insort(mdlist,(nextstart,nextend))
        next=next+relativedelta(months=+1)
    
    return mdlist

if __name__=='__main__':
    mdp1=calc_monthly_date_range('2019-01-01','2019-06-30')
    mdp2=calc_monthly_date_range('2000-01-01','2000-06-30')   #闰年
    mdp3=calc_monthly_date_range('2018-09-01','2019-03-31')   #跨年
    
    for i in range(0,len(mdp1)):
        start=mdp1[i][0]
        end=mdp1[i][1]
        print("start =",start,"end =",end)


#==============================================================================

def plot_rar_monthly(portfolio,start,end,rar_type):
    """
    功能：将风险调整收益率和风险溢价逐月绘图对比
    输入：投资组合，开始/结束日期，风险调整收益指数类别
    输出：风险调整收益率和风险溢价的逐月数据框
    显示：按月绘图投资组合的风险调整收益率和风险溢价
    """
    #仅为测试用
    #portfolio={'Market':('US','^GSPC'),'JD':0.3,'BABA':0.7}
    #start='2019-01-01'
    #end='2019-03-31'
    #rar_type='sortino_ratio'    
    
    #各项函数参数检查
    #从字典中提取信息
    plist=[]
    for key,value in portfolio.items():
        plist=plist+[value]
    #portionlist=plist[1:]
    
    #检查1：投资组合各个成分股份额的合理性
    """
    if round(sum(portionlist),1) != 1.0:
        print("  #Error(plot_rar_monthly): Incorrect total of portions")
        return None    
    """
    
    #检查2：开始/结束日期大小
    import pandas as pd
    try:
        startdate=pd.to_datetime(start)
        enddate=pd.to_datetime(end)
    except:
        print("  #Error(plot_rar_monthly): Incorrect start/end date(s)")
        return None
    if startdate >= enddate:
        print("  #Error(plot_rar_monthly): start date should be earlier")
        return None        

    #检查3：支持的rar_type
    rar_list=['treynor_ratio','sharpe_ratio','sortino_ratio','jensen_alpha']
    if rar_type not in rar_list:
        print("  #Error(draw_rar_monthly): not supported rar type")
        print("  Supported rar type:",rar_list)              
        return None         

    #拆分start/end之间的各个年份和月份
    mdlist=calc_monthly_date_range(start,end)
    if len(mdlist) == 0:
        print("Error #5(plot_rar_monthly): start/end dates inappropriate")
        return None          

    #用于保存risk premium和rar
    print("\nCalculating monthly",rar_type,"from",start,"to",end,"......")
    rarfunc=rar_type+'_portfolio'
    rars=pd.DataFrame(columns=('YM','rp','rar'))
    for i in range(0,len(mdlist)):
        start=mdlist[i][0]
        YM=start.strftime("%Y-%m")
        print(YM,end=' ')
        end=mdlist[i][1]
        rar,rp=eval(rarfunc)(portfolio,start,end,printout=False)
        
        row=pd.Series({'YM':YM,'rp':rp,'rar':rar})
        rars=rars.append(row,ignore_index=True)         
    print("completed.")
    rars.set_index('YM',inplace=True)    

    #绘图    
    plt.plot(rars['rp'],label='risk_premium',c='blue',marker='*',ls=':',lw=3)
    plt.plot(rars['rar'],label=rar_type,c='r',lw=3,marker='o')
    plt.axhline(y=0.0,color='black',linestyle=':',lw=1) 
    titletxt="投资组合的风险调整收益"
    plt.title(titletxt)
    plt.ylabel('收益率(%)')
    plt.xticks(rotation=45)
    plt.legend(loc='best')

    import datetime as dt; today=dt.date.today() 
    footnote="数据来源：雅虎财经，"+str(today)
    plt.xlabel(footnote)   
    
    plt.show()

    return


if __name__=='__main__':
    portfolio={'Market':('US','^GSPC'),'VIPS':0.1,'PDD':0.2,'JD':0.3,'BABA':0.4}
    plot_rar_monthly(portfolio,'2019-01-01','2019-06-30','treynor_ratio') 

    portfolio={'Market':('US','^GSPC'),'AAPL':1.0}
    plot_rar_monthly(portfolio,'2017-01-01','2017-12-31','sharpe_ratio')       
#==============================================================================
#==============================================================================
def plot_rar_annual(portfolio,start,end,rar_type):
    """
    功能：将风险调整收益率和风险溢价逐年绘图对比
    输入：投资组合，开始/结束日期，风险调整收益指数类别
    输出：风险调整收益率和风险溢价的逐年数据框
    显示：按年绘图投资组合的风险调整收益率和风险溢价
    """
    #仅为测试用
    #portfolio={'Market':('US','^GSPC'),'JD':0.3,'BABA':0.7}
    #start='2013-01-01'
    #end='2018-12-31'
    #rar_type='sortino_ratio'    
    
    #各项函数参数检查
    #从字典中提取信息
    plist=[]
    for key,value in portfolio.items():
        plist=plist+[value]
    #portionlist=plist[1:]
    
    #检查1：投资组合各个成分股份额的合理性
    """
    if round(sum(portionlist),1) != 1.0:
        print("  #Error(plot_rar_annual): Incorrect total of portions")
        return None    
    """
    
    #检查2：开始/结束日期大小
    import pandas as pd
    try:
        startdate=pd.to_datetime(start)
        enddate=pd.to_datetime(end)
    except:
        print("  #Error(plot_rar_annual): Incorrect start/end date(s)")
        return None
    if startdate >= enddate:
        print("  #Error(plot_rar_annual): start date should be earlier")
        return None        

    #检查3：支持的rar_type
    rar_list=['treynor_ratio','sharpe_ratio','sortino_ratio','jensen_alpha']
    if rar_type not in rar_list:
        print("  #Error(plot_rar_annual): not supported rar type")
        print("  Supported rar type:",rar_list)              
        return None         

    #拆分start/end之间的各个年份和月份
    mdlist=calc_yearly_date_range(start,end)
    if len(mdlist) == 0:
        print("  #Error(plot_rar_annual): start/end dates inappropriate")
        return None          

    #用于保存risk premium和rar
    print("\n  Calculating yearly",rar_type,"from",start,"to",end,"......")
    rarfunc=rar_type+'_portfolio'
    rars=pd.DataFrame(columns=('YR','rp','rar'))
    for i in range(0,len(mdlist)):
        start=mdlist[i][0]
        YR=start.strftime("%Y")
        print(YR,end=' ')
        end=mdlist[i][1]
        rar,rp=eval(rarfunc)(portfolio,start,end,printout=False)
        
        row=pd.Series({'YR':YR,'rp':rp,'rar':rar})
        rars=rars.append(row,ignore_index=True)         
    print("completed.")
    rars.set_index('YR',inplace=True)    

    #绘图    
    plt.plot(rars['rp'],label='risk_premium',c='blue',marker='*',ls=':',lw=3)
    plt.plot(rars['rar'],label=rar_type,c='r',lw=3,marker='o')
    plt.axhline(y=0.0,color='black',linestyle=':',lw=1) 
    titletxt="投资组合的风险调整收益"
    plt.title(titletxt)
    plt.ylabel('收益率(%)')
    plt.xticks(rotation=45)
    plt.legend(loc='best')
    
    import datetime as dt; today=dt.date.today() 
    footnote="数据来源：雅虎财经，"+str(today)
    plt.xlabel(footnote)
    
    plt.show()

    return


if __name__=='__main__':
    portfolio={'Market':('US','^GSPC'),'VIPS':0.1,'PDD':0.2,'JD':0.3,'BABA':0.4}
    plot_rar_annual(portfolio,'2013-01-01','2019-06-30','treynor_ratio') 

    portfolio={'Market':('US','^GSPC'),'AAPL':1.0}
    plot_rar_annual(portfolio,'2015-01-01','2017-12-31','sharpe_ratio')       
#==============================================================================
# 新加入的滚动指标对比
#==============================================================================
if __name__=='__main__':
    portfolio1={'Market':('US','^GSPC'),'AAPL':1}
    portfolio2={'Market':('US','^GSPC'),'MSFT':1}
    
    start='2020-1-1'
    end  ='2020-12-31'
    scope='US'
    ratio_name='sharpe'
    window=30
    graph=True  

def compare_rar_portfolio(portfolio1,portfolio2,start,end,ratio_name='sharpe', \
                      window=240,graph=True):
    """
    功能：比较两个投资组合的风险调整收益率，并绘制曲线
    注意：无风险收益率有两个月的延迟
    """
    
    #检查日期的合理性
    result,startdate,enddate=check_period(start,end)
    if result is None:
        print("#Error(compare_rar_rolling): invalid period",start,end)
        return None       

    #检查支持的指标
    ratio_list=['treynor','sharpe','sortino','alpha']
    name_list=['特雷诺比率','夏普比率','索替诺比率','詹森阿尔法']
    if ratio_name not in ratio_list:
        message="  #Error(compare_rar_portfolio): "+"unsupported rar ratio type"
        print(message)
        return None    
    
    #计算开始日期的提前量：假定每月有20个交易日
    adjdays=int(window/20.0*30.0)+1
    import siat.common as cmn
    new_start=cmn.date_adjust(start, adjust=-adjdays)
    
    #获取第一个投资组合的数据
    rars1=rar_ratio_rolling(portfolio1,new_start,end,ratio_name=ratio_name, \
                            window=window,graph=False)
    if rars1 is None: return None
    #获取第二个投资组合的数据
    rars2=rar_ratio_rolling(portfolio2,new_start,end,ratio_name=ratio_name, \
                            window=window,graph=False)
    if rars2 is None: return None
    
    #绘制双线图
    import siat.grafix as g
    
    ticker1="投资组合1"
    colname1='RAR'
    label1=name_list[ratio_list.index(ratio_name)]
    
    ticker2="投资组合2"
    colname2='RAR'
    label2=label1    
    
    ylabeltxt=label1 
    titletxt="证券的风险调整收益（收益—风险性价比）"
    
    _,_,tickers1,shares1=decompose_portfolio(portfolio1)
    if len(tickers1) == 1:
        ticker1=tickers1[0]
        pf1str=tickers1[0]
    else:
        pf1str=ticker1+'：成分'+str(tickers1)+'，比例'+str(shares1)
    
    _,_,tickers2,shares2=decompose_portfolio(portfolio2)
    if len(tickers2) == 1:
        ticker2=tickers2[0]
        pf2str=tickers2[0]
    else:
        pf2str=ticker2+'：成分'+str(tickers2)+'，比例'+str(shares2)
        
    footnote="日期 -->"
    if len(tickers1) > 1:
        footnote=footnote+'\n'+pf1str
    if len(tickers2) > 1:
        footnote=footnote+'\n'+pf2str        
        
    import datetime as dt; today=dt.date.today() 
    source="数据来源：雅虎财经，"+str(today)    
    footnote=footnote+"\n"+source
    
    g.plot_line2(rars1,ticker1,colname1,label1, \
                 rars2,ticker2,colname2,label2, \
                 ylabeltxt,titletxt,footnote)

    #合并RAR
    import pandas as pd
    rarm=pd.merge(rars1,rars2,how='inner',left_index=True,right_index=True)
    rars=rarm[['RAR_x','RAR_y']]
    rars.rename(columns={'RAR_x':ticker1+'_'+ratio_name,'RAR_y':ticker2+'_'+ratio_name},inplace=True)
    
    return rars


if __name__=='__main__':
    pf1={'Market':('US','^GSPC'),'AAPL':1}
    pf2={'Market':('US','^GSPC'),'MSFT':1}    
    rars12=compare_rar_portfolio(pf1,pf2,'2019-11-1','2020-11-30')
    
    pfA={'Market':('China','000001.SS'),'600519.SS':1}
    pfB={'Market':('China','000001.SS'),'000858.SZ':1}
    rarsAB=compare_rar_portfolio(pfA,pfB,'2019-11-1','2020-11-30')
    
    pfbb={'Market':('US','^GSPC'),'BABA':1}
    pfjd={'Market':('US','^GSPC'),'JD':1}  
    rarsbj=compare_rar_portfolio(pfbb,pfjd,'2019-11-1','2020-11-30')
    
    pfbb={'Market':('US','^GSPC'),'BABA':1}
    pfpd={'Market':('US','^GSPC'),'PDD':1}  
    rarsbj=compare_rar_portfolio(pfbb,pfpd,'2019-11-1','2020-11-30')      

#==============================================================================






# -*- coding: utf-8 -*-

import os; os.chdir("D:/siat")
from siat import *

df_all=oef_rank_china(info_type='单位净值',fund_type='全部类型',rank=15)

df_z=oef_rank_china(info_type='单位净值',fund_type='债券型')
df_g=oef_rank_china('单位净值','股票型',rank=5)
df_e=etf_rank_china(info_type='单位净值',fund_type='全部类型',rank=10)



# -*- coding: utf-8 -*-


import os; os.chdir("S:/siat")
from siat import *
#==============================================================================
cn=economy_trend('2010-1-1','2020-1-1','China','MoM CPI',power=4,zeroline=True)
cn=economy_trend('2010-1-1','2020-1-1','China','Constant CPI',power=4)
cn_jp=compare_economy(['China','Japan'],'Constant CPI','2010-1-1','2020-1-1')


#==============================================================================
cn_usa=compare_economy(['China','USA'],'Immediate Rate','2010-1-1','2021-5-25')
cn_jp=compare_economy(['China','Japan'],'Immediate Rate','2010-1-1','2021-1-1')
cn_uk=compare_economy(['China','LIBOR'],'Immediate Rate','2010-1-1','2021-1-1')
#==============================================================================
cn_usa=compare_economy(['China','USA'],'CNP GDP Per Capita','2010-1-1','2020-1-1')
cn_usa=compare_economy(['China','USA'],'CNP GDP Per Capita','2010-1-1','2020-1-1',twinx=True)

df=pmi_china('2008-1-1','2021-3-31')


c_cn=economy_trend('2010-1-1','2020-1-1','China','Constant GDP',power=3)
internal_growth_rate(c_cn)

usa_cn=compare_economy(['USA','China'],'Constant GDP','2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(usa_cn)

jp_cn=compare_economy(['Japan','China'],'Constant GDP','2010-1-1','2020-1-1',twinx=True)

df=pmi_china('2020-1-1','2020-10-1')

cn_jp=compare_economy(['China','Japan'],'Currency Value','2000-1-1','2021-1-1')
cn_us=compare_economy(['China','USA'],'Currency Value','1993-1-1','2021-1-1')

jp_us=compare_economy(['Japan','USA'],'Currency Value','1993-1-1','2021-1-1')
internal_growth_rate(cn_jp)

cn_uk=compare_economy(['China','LIBOR'],'Immediate Rate','2000-1-1','2020-1-1')

cn=economy_trend('2000-1-1','2020-8-31','China','Exchange Rate')

cn=econ_fin_depth('2000-1-1','2020-6-30','China',power=3)

cn=compare_economy(['China','USA'],'SMC to GDP','2000-1-1','2020-1-1')

cn1=economy_security('China','1999-1-1','2019-12-31','GDP','000001.SS')
cn2=economy_security('China','1999-1-1','2019-12-31','CNP GDP','000001.SS')
cn3=economy_security('China','1999-1-1','2019-12-31','CNP GDP Per Capita','000001.SS')
cn4=economy_security('China','1999-1-1','2019-12-31','GNI','000001.SS')

df=get_econ_factors0(start,end,scope,factor)


cn_usa=compare_economy(['China','USA'],'Currency Value','2010-1-1','2021-1-1')
cn_jp=compare_economy(['China','Japan'],'Currency Value','2010-1-1','2021-5-25')

cn_sg=compare_economy(['China','Korea'],'Currency Value','2010-1-1','2021-1-1')
cn_sg=compare_economy(['China','Russia'],'Currency Value','2010-1-1','2021-1-1')
cn_sg=compare_economy(['China','India'],'Currency Value','2010-1-1','2021-1-1')


cn=compare_economy(['China','USA'],'GNP Ratio','1995-1-1','2010-1-1')
cn_usa=compare_economy(['China','USA'],'GNP Ratio','1999-1-1','2019-1-1')
cn_jp=compare_economy(['China','Japan'],'GNP Ratio','1999-1-1','2019-1-1')
us_jp=compare_economy(['USA','Japan'],'GNP Ratio','1999-1-1','2019-1-1')
cn_in=compare_economy(['China','India'],'GNP Ratio','1999-1-1','2020-1-1')




cn=economy_trend('1990-1-1','2020-1-1','China','GNI',power=3)
cn_gdp_gni=compare_economy('China',['GDP','GNI'],'1990-1-1','2020-1-1')
cn_gdp_gni=compare_economy('China',['GDP','GNI'],'2010-1-1','2020-1-1',twinx=True)


cn1=economy_security('China','1999-1-1','2019-12-31','GDP','000001.SS')
cn2=economy_security('China','1999-1-1','2019-12-31','CNP GDP','000001.SS')
cn2=economy_security('China','1999-1-1','2019-12-31','Constant GDP','000001.SS')

cn3=economy_security('China','1999-1-1','2019-12-31','CNP GDP Per Capita','000001.SS')
cn4=economy_security('China','1999-1-1','2019-12-31','GNI','000001.SS')

jp1=economy_security('Japan','1980-1-1','2019-12-31','CNP GDP','^N225')
us1=economy_security('USA','1980-1-1','2019-12-31','CNP GDP','^GSPC')
us2=economy_security('USA','1980-1-1','2019-12-31','CNP GDP','^DJI')


cn=econ_fin_depth('2000-1-1','2020-6-30','China',power=3)
cn,jp=compare_efd('2000-1-1','2020-8-31',['China','Japan'])
cn,kr=compare_efd('2000-1-1','2020-8-31',['China','Korea'])
cn,us=compare_efd('2000-1-1','2020-8-31',['China','USA'])

cn=compare_economy(['China','USA'],'SMC to GDP','2000-1-1','2020-1-1')




cn_usa=compare_economy(['China','USA'],'CNP GDP Per Capita','2010-1-1','2020-1-1')
cn_usa=compare_economy(['China','USA'],'CNP GDP Per Capita','2010-1-1','2020-1-1' ,twinx=True)
internal_growth_rate(cn_usa)

cn_jp=compare_economy(['China','Japan'],'CNP GDP Per Capita','2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn_jp)

cn_in=compare_economy(['China','India'],'CNP GDP Per Capita','2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn_in)

cn=economy_trend('2010-1-1','2021-5-25','China','Constant CPI',power=4)

cn_usa=compare_economy(['China','USA'],'Constant CPI','2010-1-1','2021-5-25')
internal_growth_rate(cn_usa)

cn_jp=compare_economy(['China','Japan'],'Constant CPI','2010-1-1','2021-5-25')
internal_growth_rate(cn_jp)

cn_ru=compare_economy(['China','Russia'],'Constant CPI','2010-1-1','2020-1-1')
internal_growth_rate(cn_ru)

cn_in=compare_economy(['China','India'],'Constant CPI','2010-1-1','2020-1-1')
internal_growth_rate(cn_in)




c_cn=economy_trend('2010-1-1','2020-1-1','China','Constant GDP',power=3)
internal_growth_rate(c_cn)

usa_cn=compare_economy(['USA','China'],'Constant GDP','2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(usa_cn)

jp_cn=compare_economy(['Japan','China'],'Constant GDP','2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(jp_cn)

in_cn=compare_economy(['India','China'],'Constant GDP','2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(in_cn)

cn_gdp_c=economy_trend('2010-1-1','2020-1-1','China','Constant GDP')
internal_growth_rate(cn_gdp_c)

cn_gdp_cp=economy_trend('2010-1-1','2020-1-1','China','GDP')
internal_growth_rate(cn_gdp_cp)


cn_gdp_cnp=economy_trend('2010-1-1','2020-1-1','China','CNP GDP')
internal_growth_rate(cn_gdp_cnp)




cn=economy_trend('2010-1-1','2020-1-1','China','GDP',power=3)
internal_growth_rate(cn)

cn_cnp=compare_economy('China',['GDP','CNP GDP'],'2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn_cnp)

cn_cpi=compare_economy('China',['GDP','Constant CPI'],'2010-1-1','2020-1-1',power=4,twinx=True)
internal_growth_rate(cn_cpi)

cn_c=economy_trend('2010-1-1','2020-1-1','China','Constant GDP',power=4)
internal_growth_rate(cn_c)




cn_usa=compare_economy(['China','USA'],'Constant CPI','2010-1-1','2020-1-1')
internal_growth_rate(cn_usa)

cn=economy_trend('2010-1-1','2020-1-1','China','Constant CPI',power=4,zeroline=True)
internal_growth_rate(cn)


cn=economy_trend('2010-1-1','2020-1-1','China','MoM CPI',power=4,zeroline=True)


cn_usa=compare_economy(['China','USA'],'Constant GDP','2010-1-1','2019-1-1',twinx=True)
internal_growth_rate(cn_usa)





cn_usa=compare_economy(['China','USA'],'Constant GDP Per Capita','2010-1-1','2020-1-1')
internal_growth_rate(cn_usa)

cn_jp=compare_economy(['China','Japan'],'Constant GDP Per Capita','2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn_jp)

cn_india=compare_economy(['China','India'],'Constant GDP Per Capita','2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn_india)

cn_usa=compare_economy(['China','USA'],'Constant CPI','2010-1-1','2020-1-1')
internal_growth_rate(cn_usa)

cn_jp=compare_economy(['China','Japan'],'Constant CPI','2010-1-1','2020-1-1')
internal_growth_rate(cn_jp)

cn_ru=compare_economy(['China','Russia'],'Constant CPI','2010-1-1','2020-1-1')
internal_growth_rate(cn_ru)

cn_in=compare_economy(['China','India'],'Constant CPI','2010-1-1','2020-1-1')
internal_growth_rate(cn_in)

cn=economy_trend('2010-1-1','2020-1-1','China','GDP',power=3)
internal_growth_rate(cn)

cn=compare_economy('China',['GDP','CNP GDP'],'2010-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn)

cn=compare_economy('China',['GDP','Exchange Rate'],'2010-1-1','2020-8-31',twinx=True)
internal_growth_rate(cn)


cn=compare_economy(['China','USA'],'Immediate Rate','2000-1-1','2020-1-1')
cn_jp=compare_economy(['China','Japan'],'Immediate Rate','2000-1-1','2020-1-1')
cn_kr=compare_economy(['China','Korea'],'Immediate Rate','2000-1-1','2020-1-1')
cn_uk=compare_economy(['China','LIBOR'],'Immediate Rate','2000-1-1','2020-1-1')

cn=economy_trend('2000-1-1','2021-2-27','China','Exchange Rate')
euro=economy_trend('2000-1-1','2020-8-30','Euro','Exchange Rate',power=4)



cn1=compare_economy('China',['M0','M1'],'2000-1-1','2020-1-1')
internal_growth_rate(cn1)
df=economy_trend('2010-1-1','2020-1-1','China','M1',power=4)

cn=compare_economy('China',['M1','M2'],'2000-1-1','2020-1-1')
internal_growth_rate(cn)

cn=compare_economy('China',['M2','M3'],'2000-1-1','2020-1-1')
internal_growth_rate(cn)

cn=compare_economy('Japan',['M2','M3'],'2000-1-1','2018-1-1')
internal_growth_rate(cn)

cn_usa=compare_economy(['China','USA'],'M2','2000-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn_usa)

cn_jp=compare_economy(['China','Japan'],'M2','2000-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn_jp)

cn_kr=compare_economy(['China','Korea'],'M2','2000-1-1','2020-1-1',twinx=True)
internal_growth_rate(cn_kr)


cn=economy_trend('2000-1-1','2020-1-1','China','Discount Rate',power=0)






df=pmi_china('2020-1-1','2021-2-26')

df=pmi_china('2019-1-1','2021-2-26')

cn=economy_trend('2010-1-1','2020-1-1','China','MoM CPI',power=4,zeroline=True)

cn=economy_trend('2010-1-1','2020-1-1','China','Constant CPI',power=4)
cn=compare_economy(['China','USA'],'Constant CPI','2010-1-1','2020-1-1')
cn=compare_economy(['China','Japan'],'Constant CPI','2010-1-1','2020-1-1')
cn=compare_economy(['China','Russia'],'Constant CPI','2010-1-1','2020-1-1')
cn=compare_economy(['China','India'],'Constant CPI','2010-1-1','2020-1-1')

n=economy_trend('2010-1-1','2015-1-1','China','YoY PPI',power=4,zeroline=True)
n=economy_trend('2010-1-1','2015-1-1','China','Constant PPI',power=4,zeroline=True)




cn=economy_trend('2010-1-1','2020-1-1','China','GDP',power=3)
cn=compare_economy('China',['GDP','CNP GDP'],'2010-1-1','2020-1-1',twinx=True)

cn=compare_economy('China',['GDP','CPI'],'2010-1-1','2020-8-31',power=4,twinx=True)

cn=compare_economy('China',['GDP','Exchange Rate'],'2010-1-1','2020-8-31',twinx=True)

cn=economy_trend('2010-1-1','2020-1-1','China','Constant GDP',power=4)

cn=compare_economy(['China','USA'],'Constant GDP','2010-1-1','2020-1-1',twinx=True)

cn=compare_economy(['China','Japan'],'Constant GDP','2010-1-1','2020-1-1',twinx=True)


cn=compare_economy(['China','India'],'Constant GDP','2010-1-1','2020-1-1',twinx=True)

cn=compare_economy(['China','USA'],'Constant GDP Per Capita','2010-1-1','2020-1-1')

cn=compare_economy(['China','USA'],'Constant GDP Per Capita','2010-1-1','2020-1-1' ,twinx=True)

cn=compare_economy(['China','Japan'],'Constant GDP Per Capita','2010-1-1','2021-2-26',twinx=True)
cn=compare_economy(['China','Japan'],'Constant GDP Per Capita','2010-1-1','2021-2-26')
cn=economy_trend('2010-1-1','2020-1-1','China','Constant GDP Per Capita')
cn=economy_trend('2010-1-1','2020-1-1','Japan','Constant GDP Per Capita')
cn=economy_trend('2010-1-1','2020-1-1','USA','Constant GDP Per Capita')

cn=compare_economy(['China','India'],'Constant GDP Per Capita','2010-1-1','2020-1-1',twinx=True)

cn=compare_economy(['China','USA'],'GNP Ratio','1995-1-1','2010-1-1')

cn=compare_economy(['China','Japan'],'GNP Ratio','1995-1-1','2010-1-1')

cn=compare_economy(['USA','Japan'],'GNP Ratio','1995-1-1','2010-1-1')
cn2=compare_economy(['USA','Japan'],'GNP Ratio','1995-1-1','2010-12-31')




cn=compare_economy(['China','USA'],'GNP','1995-1-1','2020-1-1')
cn=compare_economy(['China','Japan'],'GNP','1995-1-1','2010-1-1')
cn=compare_economy(['USA','Japan'],'GNP','1995-1-1','2010-1-1')


df=pmi_china('2020-1-5','2020-10-1')

df=pmi_china('2019-1-5','2020-10-31')




# -*- coding: utf-8 -*-

start='2018-1-1'; end='2018-12-31'

import quandl
import quandl as qd
quandl.ApiConfig.api_key = "jPxsQBSvfxF_qhzTdydi"

# WTI Crude Oil price from the US Department of Energy
df1 = qd.get("EIA/PET_RWTC_D",start_date='2018-1-1',end_date='2021-10-31')

# 仅提取月底数值
df2 = qd.get("EIA/PET_RWTC_D",start_date='2018-1-1',end_date='2021-10-31', collapse="monthly")

# company profile and basic financial info
df3 = qd.get_table('ZACKS/FC', ticker='AAPL')

df4 = qd.get_table('ZACKS/FC', paginate=True)

df5 = quandl.get_table('ZACKS/FC', paginate=True, ticker='AAPL', qopts={'columns': ['ticker', 'per_end_date']})

df6 = quandl.get_table('ZACKS/FC', paginate=True, ticker=['AAPL', 'MSFT'], per_end_date={'gte': '2015-01-01'}, qopts={'columns':['ticker', 'per_end_date']})


df7=quandl.get('WIKI/AAPL',start_date='2018-1-1',end_date='2021-10-31')
#==============================================================================
import os
os.environ['QUANDL_API_KEY']="jPxsQBSvfxF_qhzTdydi" 

import pandas_datareader as web
df11=web.DataReader('HKEX/00700','quandl','2018-1-1','2021-10-31')
df12=web.DataReader('BSE/BOM532540','quandl','2018-1-1','2021-10-31')

# S&P Composite
df13=web.DataReader('YALE/SPCOMP','quandl','2018-1-1','2021-10-31')
df13b = quandl.get('YALE/SPCOMP',start_date='2018-1-1',end_date='2021-10-31')

quandl.ApiConfig.api_key = "jPxsQBSvfxF_qhzTdydi"
df13 = quandl.get('WIKI/AAPL',start_date='2018-1-1',end_date='2021-10-31', api_key="jPxsQBSvfxF_qhzTdydi") 
